import os
import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
from BASE import *

#===============================================================================#	
# Digital Input Functions	                                                   	#
#===============================================================================#
def getDIN(addr,bit):
    VerifyADDR(addr)	
    VerifyBIT(bit)
    bit -= 1
    resp=ppCMD(addr,0x20,bit,0,1)
    if resp[0] > 0:
        return 1
    else:
        return 0
		
def getDINall(addr):
    VerifyADDR(addr)	
    resp=ppCMD(addr,0x25,0,0,1)
    return resp[0]  
    # _______________________________________________________
    #|chan 8|chan 7|chan 6|chan 5|chan 4|chan 3|chan 2|chan 1|
    # -------------------------------------------------------

#==============================================================================#	
# Digital Output Functions	                                                   #
#==============================================================================#	
def setDOUT(addr,bit):
    VerifyADDR(addr)
    VerifyBIT(bit)	
    bit -= 1
    resp=ppCMD(addr,0x26,bit,0,0)
	
def clrDOUT(addr,bit):
    VerifyADDR(addr)	
    VerifyBIT(bit)
    bit -= 1
    resp=ppCMD(addr,0x27,bit,0,0)		

def toggleDOUT(addr,bit):
    VerifyADDR(addr)	
    VerifyBIT(bit)
    bit -= 1
    resp=ppCMD(addr,0x28,bit,0,0)		
	
def setDOUTall(addr,byte):
    VerifyADDR(addr)	
    assert ((byte>=0) and (byte<=255)),'ERROR: byte argument out of range - must be between 0 and 255'
    resp=ppCMD(addr,0x29,byte,0,0)			

def getDOUTall(addr):
    VerifyADDR(addr)	
    resp=ppCMD(addr,0x25,0,0,1)
    return resp[0]
    
def VerifyBIT(bit):
    assert ((bit>0) and (bit<9)), 'bit argument must be between 1 and 8'


import spidev
import time
import string
import site
import sys
from numbers import Number
import RPi.GPIO as GPIO
from six.moves import input as raw_input
GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (2,7,0)):
    sys.stderr.write("You need at least python 2.7.0 to use this library")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
DAQC2baseADDR=32
ppFRAME = 25
ppINT = 22
ppACK = 23
GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
spi = spidev.SpiDev()
spi.open(0,1)	
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/DAQC2help.txt'
#helpPath='DAQC2help.txt'       #for development only

####################################################################################
#Version    Description
#  1.0      Initial release
#  1.1      Improved download of oscilloscope data to address issues with latest OS
####################################################################################
DAQC2version=1.1

DataGood=False

RMAX = 2000
MAXADDR=8

daqc2sPresent = list(range(8))
calScale=[[0 for z in range(8)] for x in range(8)]   #16 bit signed slope calibration values - range is +/-4%
calOffset=[[0 for z in range(8)] for x in range(8)]  #16 bit signed offset calibration values - range is +/-0.1
calDAC=[[0 for z in range(8)] for x in range(8)] #16 bit signed DAC calibration values - range is +/-4%
calSet=list(range(8))
PWMvals=[[0,0],[0,0],[0,0],[0,0],[0,0],[0,0],[0,0],[0,0]]

def CLOSE():
	spi.close()
	GPIO.cleanup()

def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")

#==============================================================================#	
# A2D Functions	     	                                              		   #
#==============================================================================#	
def getADC(addr,channel):
    VerifyADDR(addr)
    VerifyAINchannel(channel)
	
    resp=ppCMD(addr,0x30,channel,0,2)
    value=(256*resp[0]+resp[1])
    if (channel==8):
        value=value*5.0*2.4/65536
    else:
        value=(value*24.0/65536)-12.0
        value=round(value*calScale[addr][channel]+calOffset[addr][channel],3)
    return value

def getADCall(addr):
    value=list(range(8))
    VerifyADDR(addr)    
    resp=ppCMD(addr,0x31,0,0,16)
    for i in range (0,8):
        value[i]=(256*resp[2*i]+resp[2*i+1])
        value[i]=(value[i]*24.0/65536)-12.0
        value[i]=round(value[i]*calScale[addr][i]+calOffset[addr][i],3)
    return value    
            
#==============================================================================#	
# DAC Output Functions	                                                       #
#==============================================================================#	
def setDAC(addr,channel,value):
    VerifyADDR(addr)
    assert (value>=0 and value<=4.095), "ERROR: DAC argument out of range - must be between 0 and 4.095 volts"
    assert (channel>=0 or channel<=3), "Error: DAC channel must be in the range of 0 to 3"
    value = int(value*calDAC[addr][channel]*1000)
    if (value>4095):
        value=4095
    hibyte = value>>8
    lobyte = value - (hibyte<<8)
    resp=ppCMD(addr,0x40+channel,hibyte,lobyte,0)
	
def getDAC(addr,channel):
    global Vcc
    VerifyADDR(addr)
    assert (channel>=0 or channel<=3), "Error: DAC channel must be in the range of 0 to 3"
    ## Return DAC value
    resp=ppCMD(addr,0x40+channel+4,0,0,2)
    value=(256*resp[0]+resp[1])
    value=value/1000.0
    return value
            
#==============================================================================#	
# Digital Output Functions	                                                   #
#==============================================================================#	
def setDOUTbit(addr,bit):
    VerifyADDR(addr)
    VerifyDOUTchannel(bit)
    resp=ppCMD(addr,0x10,bit,0,0)
	
def clrDOUTbit(addr,bit):
    VerifyADDR(addr)
    VerifyDOUTchannel(bit)
    resp=ppCMD(addr,0x11,bit,0,0)		

def toggleDOUTbit(addr,bit):
    VerifyADDR(addr)
    VerifyDOUTchannel(bit)
    resp=ppCMD(addr,0x12,bit,0,0)		
	
def setDOUTall(addr,byte):
    VerifyADDR(addr)
    assert ((byte>=0) and (byte<=255)),"Digital output value out of range. Must be in the range of 0 to 255"
    resp=ppCMD(addr,0x13,byte,0,0)			

def getDOUTbyte(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x14,0,0,1)
    return resp        

#===============================================================================#	
# Digital Input Functions	                                                   	#
#===============================================================================#
def getDINbit(addr,bit):
    VerifyADDR(addr)
    VerifyDINchannel(bit)
    resp=ppCMD(addr,0x20,bit,0,1)
    if resp[0] > 0:
        return 1
    else:
        return 0
		
def getDINall(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x25,0,0,1)
    return resp[0]

def enableDINint(addr, bit, edge):	# enable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit)	
    if ((edge=='f') or (edge=='F')):
        resp=ppCMD(addr,0x21,bit,0,0)		
    if ((edge=='r') or (edge=='R')):
        resp=ppCMD(addr,0x22,bit,0,0)
    if ((edge=='b') or (edge=='B')):
        resp=ppCMD(addr,0x23,bit,0,0)		
		
def disableDINint(addr,bit):	# disable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit)
    resp=ppCMD(addr,0x24,bit,0,0)    

#===============================================================================#	
# Interrupt Functions	                                                   		    #
#===============================================================================#	
def intEnable(addr):	#DAQC2 will pull down on INT pin if an enabled event occurs
    VerifyADDR(addr)
    resp=ppCMD(addr,0x04,0,0,0)
	
def intDisable(addr):   #DAQC2 will not assert interrupts
    VerifyADDR(addr)
    resp=ppCMD(addr,0x05,0,0,0)
    
def getINTflags(addr):	#read INT flag register in DAQC2 - this clears interrupt line and the register
    VerifyADDR(addr)
    resp=ppCMD(addr,0x06,0,0,2)
    value=(256*resp[0]+resp[1])
    return value
    
#===============================================================================#	
# LED Functions	                                                                #
# Valid led values are: off, red, yellow, green, cyan, blue, magenta, and white.#
#===============================================================================#			
LEDcolors=['off','red','green','yellow','blue','magenta','cyan','white']

def setLED(addr,led):
    VerifyADDR(addr)
    value=VerifyLED(led)
    resp=ppCMD(addr,0x60,value,0,0)

def getLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x63,0,0,1)
    return LEDcolors[resp[0]]

def VerifyLED(led):
    flag=False
    for i in range(8):
        if (led == LEDcolors[i]):
            flag=True
            value=i
    assert (flag),"-"+led + "- is not a valid LED color"
    return value
#==============================================================================#	
# Oscilloscope Functions                                                       #
#==============================================================================#   
# Shadow Registers with Defaults
#cCount=1
C1state=1
C2state=0
sRate=9
trace1=list(range(1024))
trace2=list(range(1024))
OSCtriggerChan=0   
OSCtriggerType=0
OSCtriggerEdge=0
OSCtriggerLevel=0 

def startOSC(addr):
    VerifyADDR(addr)
    #cmd = 1
    resp=ppCMD(addr,0xA1,0,0,0)
    
def stopOSC(addr):
    VerifyADDR(addr)
    #cmd = 0
    resp=ppCMD(addr,0xA0,0,0,0)
    
def runOSC(addr):
    VerifyADDR(addr)
    #cmd = 5
    resp=ppCMD(addr,0xA5,0,0,0)    
      
def setOSCchannel(addr, C1, C2):
    global C1state, C2state
    #cmd=2
    VerifyADDR(addr) 
    assert (C1==1 or C1==0),"Invalid Channel One State. Must be 0 or 1"
    assert (C2==1 or C2==0),"Invalid Channel Two State. Must be 0 or 1"
    C1state=C1
    C2state=C2  
    #assert (channelCount>=1 and channelCount<=2),"Invalid Channel Count. Must be between 1 and 2"
    #cCount=channelCount
    #resp=ppCMD(addr,0xA2,channelCount-1,0,0)
    resp=ppCMD(addr,0xA2,C1,C2,0)
    
def setOSCsweep(addr,rate):
    global sRate
    VerifyADDR(addr) 
    #cmd=3
    #rate is in range of 0 to 12
    # the following values are used by the microprocessor
    #rates: samples/sec    time/div     Timer Relod Value
    # 0:    100             1sec              0x15A0
    # 1:    200             500msec           0x8AD0
    # 2:    500             200msec           0xD120
    # 3:    1000            100msec           0xE890
    # 4:    2000            50msec            0xF448
    # 5:    5000            20msec            0xFB50 
    # 6:    10,000          10msec            0xFDA8            
    # 7:    20,000          5msec             0xFED4
    # 8:    50,000          2msec             0xFF88
    # 9:    100,000         1msec             0xFFC4
    # 10:   200,000         500usec           0xFFE2
    # 11:   500,000         200usec           0xFFF4
    # 12:   1,000,000       100usec           0xFFFA           
    # note that #12 is only valid for a single channel input
    assert ((rate>=0) and (rate<=12)),"Sweep rate value out of range. Must be in the range of 0 to 12"
    sRate=rate
    resp=ppCMD(addr,0xA3,rate,0,0)			


def getOSCtraces(addr):  
    global C1state, C2state
    cCount=C1state+C2state
    VerifyADDR(addr)
    resp=ppCMDosc(addr,0xA4,0,0,cCount*2048)
    if (cCount==2):
        for i in range(1024):
            trace1[i]=resp[4*i]*256+resp[4*i+1]
            trace2[i]=resp[4*i+2]*256+resp[4*i+3]
    else:
        if(C1state):
            for i in range(1024):
                trace1[i]=resp[2*i]*256+resp[2*i+1]
        else:
            for i in range(1024):
                trace2[i]=resp[2*i]*256+resp[2*i+1]            
    
    
def setOSCtrigger(addr,channel,type,edge,level):    
#channels: 1 or 2    0 or 1
#type: normal or auto   0 or 1
#edge: rising or falling 0 or 1
#level: 12 bit value in range of +/-12V (0 to 4095)
    options=0
    VerifyADDR(addr)
    assert (channel>=1 and channel<=2),"Invalid Channel value. Must be between 1 and 2"
    option=128*(channel-1)
    type = type.lower()
    assert (type=='auto' or type=='normal'),"Invalid trigger type. Must be 'Auto' or 'Normal'"
    if (type=='normal'):
        option += 64
    edge = edge.lower() 
    assert (edge=='rising' or edge=='falling'),"Invalid trigger edge. Must be 'rising' or 'falling'"
    if (edge=='falling'):
        option += 32
    assert (level>=0 and level <=4095),"Invalid trigger level. Must be between 0 and 4095"     
    resp=ppCMD(addr,0xA6,option+(level>>8),level&0xFF,0)
        

def trigOSCnow(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xA7,0,0,0)
     
#==============================================================================#	
# Function Generator                                                           #
#==============================================================================#     
def fgON(addr,chan):
    VerifyADDR(addr)
    VerifyFGchannel(chan)
    resp=ppCMD(addr,0x91,chan-1,0,0)

def fgOFF(addr,chan):
    VerifyADDR(addr)
    VerifyFGchannel(chan)
    resp=ppCMD(addr,0x90,chan-1,0,0)    
    
def fgFREQ(addr,chan,freq):
    VerifyADDR(addr)
    VerifyFGchannel(chan)
    assert ((freq>=10) and (freq<=20000)),"Function Generator frequency value out of range. Must be in the range of 10 to 20,000"    
    # SAMPLE_RATE_DAC 100000L        // DAC sampling rate in Hz
    # PHASE_PRECISION 65536          // Range of phase accumulator
    # FREQUENCY       FREQ           // Frequency of output in Hz
    # PHASE_ADD = FREQUENCY * PHASE_PRECISION / SAMPLE_RATE_DAC
    phase_adder=int(freq*65536/100000+0.5)
    resp=ppCMD(addr,0x92+chan-1,phase_adder>>8,phase_adder&0xFF,0) 
  
def fgTYPE(addr,chan,type):
    VerifyADDR(addr)
    VerifyFGchannel(chan)
    assert ((type>=1) and (type<=7)),"Function Generator output type is out of range. Must be in the range of 1 to 7"    
    #types:
    #   1)  sine
    #   2)  triangle
    #   3)  square
    #   4)  sawtooth rising
    #   5)  sawtooth falling
    #   6)  noise
    #   7)  sinc
    resp=ppCMD(addr,0x96,chan-1,type-1,0)
    
def fgLEVEL(addr,chan,level):
    VerifyADDR(addr)
    VerifyFGchannel(chan)
    assert ((level>=1) and (level<=4)),"Function Generator output level is out of range. Must be in the range of 1 to 4"    
    resp=ppCMD(addr,0x97,chan-1,level,0)

#==============================================================================#	
# Stepper Motor Functions                                                      #
#==============================================================================#    
def motorENABLE(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xB1,0,0,0)
    
def motorDISABLE(addr):
    VerifyADDR(addr) 
    resp=ppCMD(addr,0xB0,0,0,0)    

def motorMOVE(addr,motor,steps):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2" 
    assert ((steps>=-16383) and (steps<=16383)),"Step count is out of range. Must be between -16383 and 16383"
    if (steps<0):
        stepSign=1
    else:
        stepSign=0
    steps=abs(steps)
    param1=((motor-1)<<7) + (stepSign<<6) + (steps>>8)       
    param2= steps & 0xFF
    resp=ppCMD(addr,0xB4,param1,param2,0)
    
def motorJOG(addr,motor):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2"
    resp=ppCMD(addr,0xB5,motor-1,0,0)
 
def motorSTOP(addr,motor):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2"
    resp=ppCMD(addr,0xB6,motor-1,0,0)
 
def motorDIR(addr,motor,dir):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2"   
    dir=dir.lower()
    assert ((dir=='ccw') or (dir=='cw')), "Direction must be either clockwise (cw) or counter-clockwise (ccw)"
    if (dir=='cw'):
        param2=0
    else:
        param2=1
    resp=ppCMD(addr,0xB3,motor-1,param2,0)

def motorRATE(addr,motor,rate,stepsize):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2"
    stepsize=stepsize.lower()  
    assert ((stepsize=='w') or (stepsize=='h')),"Stepsize must be either whole (w) or half (h)"
    assert ((rate>=1) and (rate<=500)),"Rate must be greater than 1 and less than 500 steps/sec"    
    rateInc=int(rate*(2**13)/1000.0+0.5)     #convert step rate to an accumulator increment
    param1=((motor-1)<<7)+(rateInc>>8)
    if (stepsize=='h'):
        param1 |= 0x40
    param2=rateInc&0xFF
    #print motor, rate, stepsize, rateInc, param1, param2
    resp=ppCMD(addr,0xB2,param1,param2,0)

def motorOFF(addr,motor):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2" 
    resp=ppCMD(addr,0xBA,motor-1,0,0)

def motorINTenable(addr,motor):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2" 
    resp=ppCMD(addr,0xB7,motor-1,0,0)
 
def motorINTdisable(addr,motor):
    VerifyADDR(addr)
    assert ((motor>=1) and (motor<=2)),"Motor number is out of range. Must be 1 or 2" 
    resp=ppCMD(addr,0xB8,motor-1,0,0)
 
#==============================================================================#	
# PWM and Frequency Counter Functions	                                       #
#==============================================================================# 
def getFREQ(addr):
    VerifyADDR(addr)
    freq=0
    resp=ppCMD(addr,0xC0,0,0,2) #get the upper 16 bits
    if(DataGood):
        counts=(resp[0]<<24)+(resp[1]<<16)
        resp=ppCMD(addr,0xC0,1,0,2) #get the lower 16 bits
        if (DataGood):
            counts=counts+(resp[0]<<8)+resp[1]
            if (counts>0):
                freq=6000000.0/counts
    return round(freq,2)
    
def setPWM(addr,chan,dutyCycle):
    global PWMvals
    VerifyADDR(addr)
    assert ((chan>=0) and (chan<=1)),"PWM Channel number is out of range. Must be 0 or 1"
    assert ((dutyCycle>=0)and(dutyCycle<=100)), "Duty Cycle must be a value between 0 and 100"
    registerVal=int(dutyCycle*1023/100+0.5)
    param1=((chan)<<7)+(registerVal>>8)
    param2=registerVal&0xFF
    resp=ppCMD(addr,0xC1,param1,param2,0)
    PWMvals[addr][chan]=dutyCycle
    
def getPWM(addr,chan):
    global PWMvals
    VerifyADDR(addr)
    assert ((chan>=0) and (chan<=1)),"PWM Channel number is out of range. Must be 0 or 1"
    return PWMvals[addr][chan]
      
#==============================================================================#	
# System Functions	                                                           #
#==============================================================================#
def getFWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0
	
def getHWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0	

def getVersion():
    return DAQC2version   

def setINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF4,0,0,0)

def clrINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF5,0,0,0)
	
def getID(addr):
    global DAQC2baseADDR
    VerifyADDR(addr)
    addr=addr+DAQC2baseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    ppFRAME = 25
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,500000,50)
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True):
        count=0 
        csum=0
        go=True
        while (go): 
            dummy=spi.xfer([00],500000,40)
            if (dummy[0] != 0):
                num = dummy[0]
                csum += num
                id = id + chr(num)
                #print count, num
                count += 1
            else:
                dummy=spi.xfer([00],500000,40)  
                checkSum=dummy[0]                
                go=False 
            if (count>25):
                go=False
                DataGood=False
        #print checkSum, ~checkSum & 0xFF, csum & 0xFF
        if ((~checkSum & 0xFF) != (csum & 0xFF)):
            DataGood=False
    GPIO.output(ppFRAME,False)
    return id   

def getMode(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x08,0,0,1)
    return resp[0]

    
#==============================================================================#	
# Flash Memory Functions - used for calibration constants                      #
#==============================================================================#
def CalGetByte(addr,ptr):
    VerifyADDR(addr)
    assert ((ptr>=0) and (ptr<=255)),"Calibration pointer is out of range. Must be in the range of 0 to 255" 
    resp=ppCMD(addr,0xFD,2,ptr,1)
    return resp[0]
    
def CalPutByte(addr,data):
    VerifyADDR(addr)
    assert ((data>=0) and (data<=255)),"Calibration value is out of range. Must be in the range of 0 to 255"
    resp=ppCMD(addr,0xFD,1,data,0)
    
def CalEraseBlock(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xFD,0,0,0)
    
#==============================================================================#	
# LOW Level Functions	                                                       #
#==============================================================================#          
def VerifyAINchannel(ain):
    assert ((ain>=0) and (ain<=8)),"Analog input channel value out of range. Must be in the range of 0 to 8"    


def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"DAQC2plate address out of range"
    addr_str=str(addr)
    assert (daqc2sPresent[addr]==1),"No DAQC2plate found at address "+addr_str

def VerifyDOUTchannel(dout):
    assert ((dout>=0) and (dout<=7)),"Digital output channel value out of range. Must be in the range of 0 to 7"
    
def VerifyDINchannel(din):
    assert ((din>=0) and (din<=7)),"Digital input channel value out of range. Must be in the range of 0 to 7"

def VerifyFGchannel(chan):
    assert ((chan>=1) and (chan<=4)),"Function Generator channel value out of range. Must be in the range of 1 to 4"
    
def ppCMD(addr,cmd,param1,param2,bytes2return):
    global DAQC2baseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = []
    arg[0]=addr+DAQC2baseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    GPIO.output(ppFRAME,True)       #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)     #Send out 4 byte command - ignore what comes back 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):  #wait 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            #time.sleep(.0001)
            for i in range(0,bytes2return+1):	#Fetch each byte. That [00] is simply a single element list set to zero
                dummy=spi.xfer([00],500000,5)   #That [00] is simply a single element list set to zero
                resp.append(dummy[0])           
            csum=0;                             
            for i in range(0,bytes2return):     #calculate and verify checksum
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    GPIO.output(ppFRAME,False)
    return resp
    
def ppCMDosc(addr,cmd,param1,param2,bytes2return):
    global DAQC2baseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = [0]*(bytes2return)
    arg[0]=addr+DAQC2baseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    GPIO.output(ppFRAME,True)       #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)     #Send out 4 byte command - ignore what comes back 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):  #wait 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            resp=spi.xfer([0]*(bytes2return),2000000,0) 
    GPIO.output(ppFRAME,False)
    return resp
    
def getADDR(addr):
    global DAQC2baseADDR
    resp=ppCMD(addr,0x00,0,0,1)
    if (DataGood):
        return resp[0]-DAQC2baseADDR
    else:
        return 8
    
def quietPoll():   
    global daqc2sPresent
    ppFoundCount=0
    for i in range (0,8):
        daqc2sPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            daqc2sPresent[i]=1
            ppFoundCount += 1
            getCalVals(i)
            #RESET(i)


# Function to pull calibration data from flash memory
# data consists of 16 signed integers with each integer pair stored as:
# slope0,offset0,DACslope0,slope1,offset1,DACslope1,...slope7,offset,DACslope7    
# note that DACslope4 thru 7 are simply placeholders    
def getCalVals(addr):
    global calScale
    global calOffset
    values=list(range(6))
    for i in range(8):
        for j in range(6):
            values[j]=CalGetByte(addr,6*i+j)
        cSign=values[0]&0x80
        calScale[addr][i]=0.04*((values[0]&0x7F)*256+values[1])/32767   #16 bit signed slope calibration values - range is +/-4%
        if (cSign != 0):
            calScale[addr][i] *= -1
        calScale[addr][i]+=1    
        cSign=values[2]&0x80
        calOffset[addr][i]=0.2*((values[2]&0x7F)*256+values[3])/32767   #16 bit signed offset calibration values - range is +/- 0.1
        if (cSign != 0):
            calOffset[addr][i] *= -1
        cSign=values[4]&0x80
        calDAC[addr][i]=0.04*((values[4]&0x7F)*256+values[5])/32767  #16 bit signed DAC calibration values - range is +/-4%
        if (cSign != 0):
            calDAC[addr][i] *= -1
        calDAC[addr][i] += 1            

def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x0F,0,0,0)    
    time.sleep(.10)

quietPoll()    



import spidev
import time
import string
import site
import sys
from numbers import Number
import RPi.GPIO as GPIO
from six.moves import input as raw_input

GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (2,7,0)):
    sys.stderr.write("You need at least python 2.7.0 to use this library")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
RELAYbaseADDR=24
ppFRAME = 25

GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary

try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")
	
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/RELAYhelp.txt'
#helpPath='RELAYhelp.txt'       #for development only
RPversion=1.2
# Version 1.0   -   initial release
# Version 1.1 - adjusted timing on command functions to compensate for RPi SPI changes
# Version 1.2 - removed all code associated with interrupts
RMAX = 2000
MAXADDR=8
relaysPresent = list(range(8))

#==============================================================================#
# HELP Functions	                                                           #
#==============================================================================#
def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")

def getVersion():
    return RPversion


#==============================================================================#
# RELAY Functions	                                                           #
#==============================================================================#
def relayON(addr,relay):
    VerifyADDR(addr)
    VerifyRELAY(relay)
    ppCMDr(addr,0x10,relay,0,0)

def relayOFF(addr,relay):
    VerifyADDR(addr)
    VerifyRELAY(relay)
    ppCMDr(addr,0x11,relay,0,0)
    
def relayTOGGLE(addr,relay):
    VerifyADDR(addr)
    VerifyRELAY(relay)
    ppCMDr(addr,0x12,relay,0,0)   

def relayALL(addr,relays):
    VerifyADDR(addr)
    assert ((relays>=0) and (relays<=127)),"Argument out of range. Must be between 0 and 127"
    ppCMDr(addr,0x13,relays,0,0)     
 
def relaySTATE(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x14,0,0,1) 
    return resp[0]

#==============================================================================#	
# LED Functions	                                                               #
#==============================================================================#   
def setLED(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x60,0,0,0)

def clrLED(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x61,0,0,0)

def toggleLED(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x62,0,0,0)
    
#==============================================================================#	
# SYSTEM Functions	                                                           #
#==============================================================================#     
def getID(addr):
    global RELAYbaseADDR
    VerifyADDR(addr)   
    addr=addr+RELAYbaseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    ppFRAME = 25
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,300000,60)
    #null = spi.writebytes(arg)
    count=0
#    time.sleep(.01)
    while (count<20): 
        dummy=spi.xfer([00],500000,20)
        if (dummy[0] != 0):
            num = dummy[0]
            id = id + chr(num)
            count = count + 1
        else:
            count=20
    GPIO.output(ppFRAME,False)
    return id

def getHWrev(addr):
    global RELAYbaseADDR
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0	 
    
def getFWrev(addr):
    global RELAYbaseADDR
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return RPversion      
        
#==============================================================================#	
# LOW Level Functions	                                                       #
#==============================================================================#          
def VerifyRELAY(relay):
    assert ((relay>=1) and (relay<=7)),"Relay number out of range. Must be between 1 and 7"

def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"RELAYplate address out of range"
    addr_str=str(addr)
    assert (relaysPresent[addr]==1),"No RELAYplate found at address "+addr_str

def ppCMDr(addr,cmd,param1,param2,bytes2return):
    global RELAYbaseADDR
    arg = list(range(4))
    resp = []
    arg[0]=addr+RELAYbaseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,300000,60)
    #null = spi.writebytes(arg)
    if bytes2return>0:
        time.sleep(.0001)
        for i in range(0,bytes2return):	
            dummy=spi.xfer([00],500000,20)
            resp.append(dummy[0])
    time.sleep(.001)
    GPIO.output(ppFRAME,False)
    time.sleep(.001)
    return resp    
    
def getADDR(addr):
    global RELAYbaseADDR
    resp = []
    resp=ppCMDr(addr,0x00,0,0,1)
    return resp[0]-RELAYbaseADDR

    
def quietPoll():   
    global relaysPresent
    ppFoundCount=0
    for i in range (0,8):
        relaysPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            relaysPresent[i]=1
            ppFoundCount += 1
            #RESET(i)

def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x0F,0,0,0)    
    time.sleep(.10)

quietPoll()    



import spidev
import time
import site
import sys
import threading
import RPi.GPIO as GPIO
from six.moves import input as raw_input

GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("This library requires Python3")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
CURRENTbaseADDR=0x50
ppFRAME = 25
ppACK = 23
ppSRQ =22

GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppSRQ, GPIO.IN, pull_up_down=GPIO.PUD_UP)

try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")    
    
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/CURRENThelp.txt'
#helpPath='CURRENThelp.txt'       #for development only
version=1.0
# Version 1.0   -   initial release

RMAX = 2000
MAXADDR=8
CURRENTPresent = list(range(8))
DataGood=False
lock = threading.Lock()
lock.acquire()

#==============================================================================#
# HELP Functions                                                               #
#==============================================================================#
def Help():
    help()

def HELP():
    help()
    
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        raw_input('press \"Enter\" for more...')
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("No help file found.")

def getSWrev():
    return version

#==============================================================================#
# Current Read Functions                                                       #
#==============================================================================#
def getI(addr,channel):
    VerifyADDR(addr)
    VerifyIchannel(channel)
    resp=ppCMD(addr,0x30,channel-1,0,2)
    value=float(256*resp[0]+resp[1])
    value=round((value*24.0/65536.0),4)
    return value

def getIall(addr):
    value=list(range(8))
    VerifyADDR(addr)    
    resp=ppCMD(addr,0x31,0,0,16)
    for i in range (0,8):
        value[i]=float(256*resp[2*i]+resp[2*i+1])
        value[i]=round((value[i]*24.0/65536.0),4)
    return value 


#==============================================================================#
# Unused Precision Functions - analysis showed no clear improvement            #
#==============================================================================#
def initI(addr,channel):
    VerifyADDR(addr)
    VerifyIchannel(channel) 
    ppCMD(addr,0x32,channel-1,0,0)

def initIall(addr):
    VerifyADDR(addr)    
    ppCMD(addr,0x33,0,0,0)   
    
def pullI(addr, channel):
    VerifyADDR(addr)
    VerifyIchannel(channel) 
    resp=ppCMD(addr,0x34,channel-1,0,2)
    value=float(256*resp[0]+resp[1])
    value=round((value*24.0/65536.0),4)
    return value
    
def pullIall(addr):
    value=list(range(8))
    VerifyADDR(addr)    
    resp=ppCMD(addr,0x35,0,0,16)
    for i in range (0,8):
        value[i]=float(256*resp[2*i]+resp[2*i+1])
        value[i]=round((value[i]*24.0/65536.0),4)
    return value 
    
def setFREQ(addr,freq):
    VerifyADDR(addr)
    assert (freq==50 or freq==60), "AC line frequency argument can only be for or 60."
    ppCMD(addr,0x3F,freq,0,0)

#==============================================================================#
# LED Functions                                                                #
#==============================================================================#
def setLED(addr):
    VerifyADDR(addr)
    ppCMD(addr,0x60,0,0,0)

def clrLED(addr):
    VerifyADDR(addr)
    ppCMD(addr,0x61,0,0,0)

def toggleLED(addr):
    VerifyADDR(addr)
    ppCMD(addr,0x62,0,0,0)

#==============================================================================#    
# Interrupt Functions                                                          #
#==============================================================================#    
def intEnable(addr):    #CURRENT will pull down on SRQ pin if an enabled event occurs
    VerifyADDR(addr)
    ppCMD(addr,0x04,0,0,0)

def intDisable(addr):   #CURRENT will not assert SRQ
    VerifyADDR(addr)
    ppCMD(addr,0x05,0,0,0)
    
def getINTflags(addr):  #read SRQ flag register in CURRENT - this clears interrupt line and the register
    VerifyADDR(addr)
    resp=ppCMD(addr,0x06,0,0,1)
    value=(resp[0])
    return value

    
#==============================================================================#    
# Flash Memory Functions - used for calibration constants                      #
#==============================================================================#
def CalGetByte(addr,ptr):
    VerifyADDR(addr)
    assert ((ptr>=0) and (ptr<=255)),"Calibration pointer is out of range. Must be in the range of 0 to 255" 
    resp=ppCMD(addr,0xFD,2,ptr,1)
    return resp[0]
    
def CalPutByte(addr,data):
    VerifyADDR(addr)
    assert ((data>=0) and (data<=255)),"Calibration value is out of range. Must be in the range of 0 to 255"
    ppCMD(addr,0xFD,1,data,0)
    
def CalEraseBlock(addr):
    VerifyADDR(addr)
    ppCMD(addr,0xFD,0,0,0)    
    
#==============================================================================#    
# SYSTEM Functions                                                             #
#==============================================================================#     
def getID(addr):
    global CURRENTbaseADDR
    global ppFRAME
    global DataGood
    VerifyADDR(addr)
    addr=addr+CURRENTbaseADDR
    id=""
    arg = list(range(4))
    #resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)==1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False     
    if (DataGood==True):
        GPIO.output(ppFRAME,True)
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        spi.xfer(arg,500000,5)
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):              
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            time.sleep(0.000001)     #allow the uP some time to initialize the SPI
            count=0 
            csum=0
            go=True
            while (go): 
                dummy=spi.xfer([00],500000,5)
                if (dummy[0] != 0):
                    num = dummy[0]
                    csum += num
                    id = id + chr(num)
                    count += 1
                else:
                    dummy=spi.xfer([00],500000,1)  
                    checkSum=dummy[0]                
                    go=False 
                if (count>25):
                    go=False
                    DataGood=False
            if ((~checkSum & 0xFF) != (csum & 0xFF)):
                DataGood=False
        GPIO.output(ppFRAME,False)
        time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return id

def getHWrev(addr):
    global CURRENTbaseADDR
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0  
    
def getFWrev(addr):
    global CURRENTbaseADDR
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return version      

def setINT(addr):
    VerifyADDR(addr)
    ppCMD(addr,0xF4,0,0,0)

def clrINT(addr):
    VerifyADDR(addr)
    ppCMD(addr,0xF5,0,0,0)
        
#==============================================================================#    
# LOW Level Functions                                                          #
#==============================================================================#          
def VerifyIchannel(Iin):
    assert ((Iin>=1) and (Iin<=8)),"4-20mA input channel value out of range. Must be in the range of 1 to 8" 

def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"CURRENTplate address out of range"
    addr_str=str(addr)
    assert (CURRENTPresent[addr]==1),"No CURRENTplate found at address "+addr_str


def ppCMD(addr,cmd,param1,param2,bytes2return):
    global CURRENTbaseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = [0]*(bytes2return+1)
    arg[0]=addr+CURRENTbaseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    wait=True
    t0=time.time()
    while(wait):    #ensure that ACK is high before asserting FRAME
        if (GPIO.input(ppACK)!=0):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False     
    GPIO.output(ppFRAME,True)       #Set FRAME high - tell Pi-Plates to start listening
    spi.xfer(arg,500000,5)     #Send out 4 byte command 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):  #wait up to 50msec for the addressed plate to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05): #test for timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If plate is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood):                  #if ACK is low AND there was no timeout then fetch data
            resp=spi.xfer([0]*(bytes2return+1),500000,0)    
            csum=0;                             
            for i in range(0,bytes2return):     #calculate and verify checksum
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    GPIO.output(ppFRAME,False)
    return resp

    
def getADDR(addr):
    global CURRENTbaseADDR
    #resp = []
    resp=ppCMD(addr,0x00,0,0,1)
    if (DataGood):
        return resp[0]-CURRENTbaseADDR
    else:
        return 8

    
def quietPoll():   
    global CURRENTPresent
    global DataGood
    ppFoundCount=0
    for i in range (0,8):
        CURRENTPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            CURRENTPresent[i]=1
            ppFoundCount += 1

def RESET(addr):
    VerifyADDR(addr)
    ppCMD(addr,0x0F,0,0,0)    
    time.sleep(1.1)

quietPoll()    



import spidev
import time
import string
import site
import sys
import threading
from numbers import Number
import RPi.GPIO as GPIO
from six.moves import input as raw_input

GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("You need at least python 2.7.0 to use this library")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
RELAY2baseADDR=0x48
ppFRAME = 25
ppACK = 23

GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)

try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")	
    
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/RELAY2help.txt'
#helpPath='RELAY2help.txt'       #for development only
RP2version=1.0
# Version 1.0   -   initial release

RMAX = 2000
MAXADDR=8
relaysPresent = list(range(8))
DataGood=False
lock = threading.Lock()
lock.acquire()

#==============================================================================#
# HELP Functions	                                                           #
#==============================================================================#
def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")

def getSWrev():
    return RP2version


#==============================================================================#
# RELAY Functions	                                                           #
#==============================================================================#
def relayON(addr,relay):
    VerifyADDR(addr)
    VerifyRELAY(relay)
    ppCMDr(addr,0x10,relay-1,0,0)

def relayOFF(addr,relay):
    VerifyADDR(addr)
    VerifyRELAY(relay)
    ppCMDr(addr,0x11,relay-1,0,0)
    
def relayTOGGLE(addr,relay):
    VerifyADDR(addr)
    VerifyRELAY(relay)
    ppCMDr(addr,0x12,relay-1,0,0)   

def relayALL(addr,relays):
    VerifyADDR(addr)
    assert ((relays>=0) and (relays<=255)),"Argument out of range. Must be between 0 and 255"
    ppCMDr(addr,0x13,relays,0,0)     
 
def relaySTATE(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x14,0,0,1) 
    return resp[0]

#==============================================================================#	
# LED Functions	                                                               #
#==============================================================================#   
def setLED(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x60,0,0,0)

def clrLED(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x61,0,0,0)

def toggleLED(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x62,0,0,0)
    
#==============================================================================#	
# SYSTEM Functions	                                                           #
#==============================================================================#     
def getID(addr):
    global RELAY2baseADDR
    global ppFRAME
    VerifyADDR(addr)
    addr=addr+RELAY2baseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)==1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False     
    if (DataGood==True):
        GPIO.output(ppFRAME,True)
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        null=spi.xfer(arg,500000,5)
        #DataGood=True
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):              
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            time.sleep(0.000001)     #allow the uP some time to initialize the SPI
            count=0 
            csum=0
            go=True
            while (go): 
                dummy=spi.xfer([00],500000,5)
                if (dummy[0] != 0):
                    num = dummy[0]
                    csum += num
                    id = id + chr(num)
                    count += 1
                else:
                    dummy=spi.xfer([00],500000,1)  
                    checkSum=dummy[0]                
                    go=False 
                if (count>25):
                    go=False
                    DataGood=False
            if ((~checkSum & 0xFF) != (csum & 0xFF)):
                DataGood=False
        GPIO.output(ppFRAME,False)
        time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return id

def getHWrev(addr):
    global RELAY2baseADDR
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0	 
    
def getFWrev(addr):
    global RELAY2baseADDR
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return RP2version      
        
#==============================================================================#	
# LOW Level Functions	                                                       #
#==============================================================================#          
def VerifyRELAY(relay):
    assert ((relay>=1) and (relay<=8)),"Relay number out of range. Must be between 1 and 8"

def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"RELAY2plate address out of range"
    addr_str=str(addr)
    assert (relaysPresent[addr]==1),"No RELAY2plate found at address "+addr_str

def ppCMDr(addr,cmd,param1,param2,bytes2return,slow=None):
    global RELAY2baseADDR
    global DataGood
    if (slow==None):
        tOut=0.05
    else:
        tOut=3.0
    arg = list(range(4))
    #resp = []
    resp = [0]*(bytes2return+1)
    #dummy=resp
    arg[0]=addr+RELAY2baseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)!=0):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False 
    GPIO.output(ppFRAME,True)
    time.sleep(0.000001)     #allow the uP some time to initialize the SPI
    null=spi.xfer(arg,400000,0) 
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):
            wait=False
        if ((time.time()-t0)>tOut):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):              
                wait=False
            if ((time.time()-t0)>0.1):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            resp=spi.xfer([0]*(bytes2return+1),400000,0)
            csum=0;
            for i in range(0,bytes2return+1):
                csum+=resp[i]
            if ((csum & 0xFF) != 0xFF):
                DataGood=False
    GPIO.output(ppFRAME,False)
    time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return resp
    
def getADDR(addr):
    global RELAY2baseADDR
    resp = []
    resp=ppCMDr(addr,0x00,0,0,1)
    return resp[0]-RELAY2baseADDR

    
def quietPoll():   
    global relaysPresent
    ppFoundCount=0
    for i in range (0,8):
        relaysPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            relaysPresent[i]=1
            ppFoundCount += 1
            #RESET(i)

def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMDr(addr,0x0F,0,0,0)    
    time.sleep(.10)

quietPoll()    



import os
import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
from BASE import *

dist_units = 'in'
#==============================================================================#
# RANGEFINDER Functions	                                                       #
# Uses "channel pairs" with valid values of 12, 34, 56 and 78                  #
#==============================================================================#
def getRANGE(addr,channelpair,units=None):
    global dist_units
    VerifyADDR(addr)	
    assert ((channelpair==12) or (channelpair==34) or (channelpair==56) or (channelpair==78)),"Invalid channel pair argument. Valid values are 12, 34, 56, and 78"
    channelpair=(channelpair>>1)//10
    if units is None:
        uni=dist_units
    else:   
        uni=units.lower()
        assert ((uni=='cm') or (uni=='in')),"ERROR: incorrect units parameter. Must be 'cm' or 'in'."
    resp=ppCMD(addr,0x81,channelpair,0,2)   #get data
    Range=resp[0]*256+resp[1]
    if (Range==0):
        return "ERROR: sensor failure"
    Range=(Range<<1)*12.0/49.0
    if (uni=='cm'):
        Range = Range/58.326
    if (uni=='in'):
        Range = Range/148.148
    Range=round(Range,2)
    return Range
    
def getRANGEfast(addr,channelpair,units=None):
    global dist_units
    VerifyADDR(addr)	
    assert ((channelpair==12) or (channelpair==34) or (channelpair==56)or (channelpair==78)),"Invalid channel pair argument. Valid values are 12, 34, 56, and 78"
    channelpair=(channelpair>>1)//10
    if units is None:
        uni=dist_units
    else:   
        uni=units.lower()
        assert ((uni=='cm') or (uni=='in')),"ERROR: incorrect units parameter. Must be 'cm' or 'in'."
    resp=ppCMD(addr,0x82,channelpair,0,2)   #get data
    Range=resp[0]*256+resp[1]
    if (Range==0):
        return "ERROR: sensor failure"
    Range=(Range<<1)*12.0/49.0
    if (uni=='cm'):
        Range = Range/58.326
    if (uni=='in'):
        Range = Range/148.148
    Range=round(Range,2)
    return Range
    
def setUNITS(units):
    global dist_units
    units=units.lower()
    assert ((units=='in') or (units=='cm')), "Distance units must be 'in' or 'cm'."
    dist_units=units
    
def getUNITS():
    global dist_units
    return dist_units


import spidev
import time
import site
import sys
import threading
import RPi.GPIO as GPIO
GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("You need python 3 to use this library")
    exit(1)

Vref=2.5
FSR=25

GPIO.setmode(GPIO.BCM)
ADCbaseADDR=64
ppFRAME = 25
ppINT = 22
ppACK = 23
ppSW = 24
drMAP=[5,6,13,19,26,21,20,16]
GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppSW, GPIO.IN, pull_up_down=GPIO.PUD_UP)

try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")

localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/ADChelp.txt'
#helpPath='ADChelp.txt'       #for development only
ADCversion=1.0
DataGood=False
lock = threading.Lock()
lock.acquire()

RMAX = 2000
MAXADDR=8
SHARED=0
DEDICATED=1


adcsPresent = list(range(8))
chanTYPE=[[256 for z in range(16)] for x in range(8)]       #character to indicate sample rate for each channel
chanENABLE=[[False for z in range(16)] for x in range(8)]   #enable status for each channel
trigENABLED=[False for z in range(8)]
assignedSETUP=[[0xFF for z in range(8)] for x in range(8)]

#lastTYPE=['v' for z in range(8)]
ADCbusy=[False for z in range(8)]
ADCerr=[0 for z in range(8)]
blockSIZE=[0 for z in range(8)]
streamSIZE=[0 for z in range(8)]
streamMODE=[False for z in range(8)]
blockMODE=[False for z in range(8)]
blockBYTES=[0]*8192*4
blockVALS=[0]*8192
blockCHANS=[0]*8192
srLIST=[5 for z in range(8)]
modeLIST=[1 for z in range(8)]
scanLIST=[0 for z in range(8)]
srqSOURCE=[SHARED for z in range(8)]

def CLOSE():
    spi.close()
    GPIO.cleanup()

def Help():
    help()

def HELP():
    help()

def help():
    valid=True
    try:
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        input('press \"Enter\" for more...')
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")

def setMODE(addr,mode):
    global ADCbusy, modeLIST
    modeindex=['SLOW', 'MED', 'FAST', 'ADV','HIGH','MED','LOW','ADV']
    if (VerifyADDR(addr) == 0):
        return
    if(ADCbusy[addr]!=False):
        print("You cannot change mode while ADCplate is busy with a measurement.")
        #sys.exit()
    else:
        if (isinstance(mode, str) and mode.isalpha()):
            mode=mode.upper()
        else:
            print("Not a valid mode. Mode arguments can be 'SLOW', 'MEDium', 'FAST', or 'ADVanced'")
            print("Or mode arguments can be 'HIGH', 'MEDium', 'LOW', or 'ADVanced'")
            return            
        if (mode!='SLOW' and mode!='FAST' and mode[:3]!='MED' and mode[:3]!='ADV' and mode!='HIGH' and mode!='LOW'):
            print("Not a valid mode. Mode arguments can be 'SLOW', 'MEDium', 'FAST', or 'ADVanced'" )
            print("Or mode arguments can be 'HIGH', 'MEDium', 'LOW', or 'ADVanced'")            
            return
        ppCMD(addr,0x3B,modeindex.index(mode)%4,0,0)
        if (mode[:3]=='ADV'):
            enableEVENTS(addr)
            getEVENTS(addr)
        else:
            disableEVENTS(addr)
            getEVENTS(addr)
        modeLIST[addr]=modeindex.index(mode)%4

def getMODE(addr):
    if (VerifyADDR(addr) == 0):
        return
    resp=ppCMD(addr,0x3C,0,0,1)
    modeLIST[addr]=resp[0]
    return resp[0]


#==============================================================================#
# Begin A2D Functions - configuration
#==============================================================================#
#enableINPUT(addr,input,SR)
# input - the ADCplate supports 16 inputs labeled S0 thru S7, D0 thru D3, and I0 thru I3
#         numeric values of 00-15 can be used as well
# sampleRate(sr) - each channel can have unique sampling rate with values from
#                  0 to 18. The characteristics of each of these values is shown
#                  below:
# SR    ODR Fdata  Ts(msec)    Fmux(Hz)    fc (Hz)  50Hz Rej 60Hz Rej Venb   Ienb   Filter
# 0     22  1.25    2400        1.25        0.3     -125.63 -130.43   22.7    24      3
# 1     21  2.5     1200        2.5         0.6     -108.66 -113.49   22.7    24      3
# 2     20  5       600         5           1.3     -103.09 -107.92   22.4    24      3
# 3     19  10      300         10          2.6     -101.71 -106.52   22.4    24      3
# 4     NA  16.667  60          16.67       ~10     -90     -90       21.9    24      5
# 5     NA  20      50          20.00       ~10     -85     -85       21.8    24      5
# 6     NA  25      40          25.00       ~10     -62     -62       21.6    23.6    5
# 7     16  50      60          49.68       12.8    -100.76 -46.95    21.8    23.7    3
# 8     15  59.98   50.02       59.52       15.4    -40.34  -105.8    21.6    23.6    3
# 9     14  100.2   10          100.20      44.2    NA      NA        21.3    20.6    5
# 10    13  200.3   5           200.30      89.5    NA      NA        20.6    20.1    5
# 11    12  381     2.63        380.95      174.4   NA      NA        20.2    19.9    5
# 12    11  504     1.99        503.8       234.0   NA      NA        19.9    19.4    5
# 13    10  1007    0.993       1007.00     502     NA      NA        19.5    18.8    5
# 14    9   2597    0.385       2597.00     1664.0  NA      NA        18.7    18.0    5
# 15    8   5208    0.321       3115        2182    NA      NA        18.3    17.9    5
# 16    7   10417   0.225       4444        3933    NA      NA        17.9    17.4    5
# 17    6   15625   0.193       5181        5150    NA      NA        17.7    17.2    5
# 18    5   31250   0.161       6211       6750    NA      NA        17.5    17.0    5
#=======================================================================================#

def srTable():
    print('=================================================================')
    print('SR Fdata  Ts(msec) Fmux(Hz) fc (Hz)  50Hz Rej 60Hz Rej Venb Ienb ')
    print('0  1.25    2400     1.25     0.3     -125.63 -130.43   22.7  24  ')
    print('1  2.5     1200     2.5      0.6     -108.66 -113.49   22.7  24  ')
    print('2  5       600      5        1.3     -103.09 -107.92   22.4  24  ')
    print('3  10      300      10       2.6     -101.71 -106.52   22.4  24  ')
    print('4  16.667  60       16.67    ~10     -90     -90       21.9  24  ')
    print('5  20      50       20.00    ~10     -85     -85       21.8  24  ')
    print('6  25      40       25.00    ~10     -62     -62       21.6  23.6')
    print('7  50      60       49.68    12.8    -100.76 -46.95    21.8  23.7')
    print('8  59.98   50.02    59.52    15.4    -40.34  -105.8    21.6  23.6')
    print('9  100.2   10       100.20   44.2      NA      NA      21.3  20.6')
    print('10 200.3   5        200.30   89.5      NA      NA      20.6  20.1')
    print('11 381     2.63     380.95   174.4     NA      NA      20.2  19.9')
    print('12 504     1.99     503.8    234.0     NA      NA      19.9  19.4')
    print('13 1007    0.993    1007.00  502       NA      NA      19.5  18.8')
    print('14 2597    0.385    2597.00  1664.0    NA      NA      18.7  18.0')
    print('15 5208    0.321    3115     2182      NA      NA      18.3  17.9')
    print('16 10417   0.225    4444     3933      NA      NA      17.9  17.4')
    print('17 15625   0.193    5181     5150      NA      NA      17.7  17.2')
    print('18 31250   0.161    6211     6750      NA      NA      17.5  17.0')
    print('=================================================================')


def configINPUT(addr,input,sampleRate,enable=None):
    global chanENABLE
    global chanTYPE
    global assignedSETUP
    global ADCbusy
    global modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    if ((sampleRate < 0) or (sampleRate > 18)):
        print('Command ignored - the sample rate argument must be between 0 and 18.')
        return
    cCheck=verifyInput(input)
    cTypes=['S0','S1','S2','S3','S4','S5','S6','S7','D0','D1','D2','D3','I0','I1','I2','I3']
    mux=cTypes[cCheck]
    #chanTYPE[addr][cCheck]=mux+'-'+str('{:02d}'.format(sampleRate))   #local copy of channel configuration
    chanTYPE[addr][cCheck]=sampleRate  #local copy of channel sample rate
    if (enable==None):
        enable=chanENABLE[addr][cCheck]
    else:
        if(enable!=True and enable!=False):
            print ('Command ignored - the optional enable argument must be "True" or "False".')
            return
    if (enable):
        if (setupCOUNT(addr)>7):
            print ('Command ignored - the maximum enabled channel count (8) exceeded.')
            return
        chanENABLE[addr][cCheck]=enable
        setup=getSETUP(addr,cCheck)<<4
        eARG=3
    else:
        eARG=2
        setup=0xF0
    #muxSelect=cCheck #this is the mux selection value to to send to the ADC
    ppCMD(addr,0x39, setup+cCheck,(eARG<<5)+sampleRate,0)

def enableINPUT(addr,input):
    global chanENABLE
    global ADCbusy,modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    cCheck=verifyInput(input)
    if (setupCOUNT(addr)>7):
        print ('Command ignored - the maximum enabled channel count (8) exceeded.')
        return
    setup=getSETUP(addr,cCheck)<<4
    chanENABLE[addr][cCheck]=True
    ppCMD(addr,0x39,setup+cCheck,(1<<5),0)


def disableINPUT(addr,input):
    global chanENABLE
    global ADCbusy, modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    cCheck=verifyInput(input)
    clrSETUP(addr,cCheck)
    chanENABLE[addr][cCheck]=False
    ppCMD(addr,0x39,0xF0+cCheck,(0<<5),0)

def verifyInput(mux):
    #global chanTYPE
    if (isinstance(mux,int)):
        assert (mux<16), 'Invalid input value - must be between 0 and 15.'
        cNum=mux
    else:
        assert (len(mux)==2 and mux[0].isalpha() and mux[1].isdigit()), 'Invalid input argument - must be S0-S7, D0-D3, or I0-I3.'
        mux=mux[0].upper()+mux[1]
        cType=mux[0]
        assert (cType=='S' or cType=='D' or cType=='I'), 'Invalid input type - must be S0-S7, D0-D3, or I0-I3.'
        cNum=int(mux[1])
        if (cType=='S'):
            assert ((cNum >= 0) and (cNum <=7)), 'Only single ended input numbers 0 through 7 are valid'
            #chan=cNum
        if (cType=='D'):
            assert ((cNum >= 0) and (cNum <=3)), 'Only differential input numbers 0 through 3 are valid'
            cNum=8+cNum
        if (cType=='I'):
            assert ((cNum >= 0) and (cNum <=3)), 'Only current input numbers 0 through 3 are valid'
            cNum=12+cNum
    return cNum

def initADC(addr):
    global chanENABLE, ADCbusy, streamMODE, trigENABLED, modeLIST
    if (VerifyADDR(addr) == 0):
        return
    ppCMD(addr,0x3A,0,0,0)
    for i in range(16):
        chanENABLE[addr][i]=False
    for i in range(8):
        assignedSETUP[addr][i]=0xFF
    ADCbusy[addr]=False
    streamMODE[addr]=False
    trigENABLED[addr]=False
    modeLIST[addr] = 1
    disableEVENTS(addr)
    time.sleep(0.01)
    

def getSETUP(addr,chan):
    global assignedSETUP
    #first check to see if channel has already been assigned a setup
    i=0
    search=True
    while(search and (i<8)):
        if (assignedSETUP[addr][i]==chan):
            search=False
        else:
            i=i+1
    #if i==8 then channel has not already been assigned a setup so find the next available one
    if (i==8):
        i=0
        search=True
        while(search and (i<8)):
            if (assignedSETUP[addr][i] == 0xFF):
                assignedSETUP[addr][i]=chan
                search=False
            else:
                i=i+1
    return i

def clrSETUP(addr,chan):
    global assignedSETUP
    i=0
    search=True
    while(search and (i<8)):
        if (assignedSETUP[addr][i]==chan):
            search=False
            assignedSETUP[addr][i]=0xFF
        else:
            i=i+1
    pass

def setupCOUNT(addr):
    global assignedSETUP
    count=0
    for i in range(8):
        if (assignedSETUP[addr][i]!=0xFF):
            count=count+1
    return count

#==============================================================================#
#ADC - read  modes and functions
#==============================================================================#
def getADC(addr,input):
    global ADCbusy, modeLIST, Vref
    cCheck=verifyInput(input)
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr]==3):
        print("Command ignored - this function cannot be used in ADVanced mode")
        return
    resp=ppCMD(addr,0x30,cCheck,0,4)
    val=((resp[0])<<16)+(resp[1]<<8)+resp[2]
    assert(val<0xFFFFFF),'ADC error occurred'
    ADCerr[addr]=resp[3]
    if (cCheck<12):
        Vin=10*Vref*((val/8388608.0)-1)*FSR/25
    else:
        Vin=1000/50*Vref*val/16777216.0 #return current in mA
    ADCbusy[addr]=False
    return round(Vin,6)

def getADCall(addr):
    global Vref, modeLIST, blockSIZE, blockVALS, blockCHANS
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr]==3):
        print("Command ignored - this function cannot be used in ADVanced mode")
        return       
    blockSIZE[addr]=16
    fetchBLOCK(addr,0x31,0,0,64)
    for i in range(blockSIZE[addr]):
        k=i<<2
        val=blockBYTES[k]<<16
        val += blockBYTES[k+1]<<8
        val += blockBYTES[k+2]
        lblockCHAN=blockBYTES[k+3] & 0x0F
        if (i<12):
            Vin=10.0*Vref*((val/8388608.0)-1)*FSR/25
        else:
            Vin=1000/50*Vref*val/16777216.0 #return current in mA
        blockVALS[i] = round(Vin,7)
        blockCHANS[i]=lblockCHAN
    return blockVALS[0:blockSIZE[addr]]

def getSall(addr):  #Only return the eight Single Ended inputs
    wholeBlock=16*[0]
    wholeBlock=getADCall(addr)
    return wholeBlock[:8]

def getDall(addr):  #Only return the four Differential inputs
    wholeBlock=16*[0]
    wholeBlock=getADCall(addr)    
    return wholeBlock[8:12]

def getIall(addr):  #Only return the four 4-30mA inputs
    wholeBlock=16*[0]
    wholeBlock=getADCall(addr)    
    return wholeBlock[12:16]
    
def readSINGLE(addr,input,sampleRate=None):
    #VerifyADDR(addr)
    global ADCbusy, modeLIST
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    cCheck=verifyInput(input)
    if (sampleRate==None):
        startSINGLE(addr,input)
    else:
        startSINGLE(addr,input,sampleRate)
    while(check4EVENTS(addr)==False):
        pass
    getEVENTS(addr) #clear the interrupt register
    return getSINGLE(addr,cCheck)

def startSINGLE(addr,input,sampleRate=None):
    global ADCbusy, modeLIST
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    cCheck=verifyInput(input)
    if (sampleRate!=None):
        assert ((sampleRate >= 0) and (sampleRate <= 18)), 'The sample rate argument must be between 0 and 18.'
        sr=sampleRate
    else:
        sr=0xFF
    ADCbusy[addr]=True
    ppCMD(addr,0x32,cCheck,sr,0)

def getSINGLE(addr,input):
    global ADCerr, Vref
    global ADCbusy, modeLIST
    #assert(ADCbusy[addr]==False), "This ADCplate is busy a another measurement."
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    cCheck=verifyInput(input)
    resp=ppCMD(addr,0x33,0,0,4,'s')
    val=((resp[0])<<16)+(resp[1]<<8)+resp[2]
    assert(val<0xFFFFFF),'ADC error occurred'
    ADCerr[addr]=resp[3]
    #Vref=2.5
    if (cCheck<12):
        Vin=10*Vref*((val/8388608.0)-1)*FSR/25
    else:
        Vin=1000/50*Vref*val/16777216.0 #return current in mA
    ADCbusy[addr]=False
    return round(Vin,6)

def readSCAN(addr):
    global ADCbusy, chanENABLE, modeLIST
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    N=0
    for i in range(16):
        if (chanENABLE[addr][i]==True):
            N=N+1
    if (N>0):
        startSCAN(addr)
        while(check4EVENTS(addr)==False):
            pass
        getEVENTS(addr) #clear the interrupt register
        return getSCAN(addr)
    else:
        chanVALS=[None for z in range(16)]
        return chanVALS

def startSCAN(addr,trig=None):
    global ADCbusy, blockSIZE, modeLIST,trigENABLED
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    assert ((trig==None) or (trig==True) or (trig==False)),"If used, the optional trig value should be True or False."
    trigFlag=0
    if (trig==True):
        assert(trigENABLED[addr]==True), "The trigger settings need to be configured before use."
        trigFlag=0x80
    ADCbusy[addr]=True
    N=0
    for i in range(16):
        if (chanENABLE[addr][i]==True):
            N=N+1
    blockSIZE[addr]=N
    if (N>0):
        ppCMD(addr,0x34,trigFlag,0,0)

def getSCAN(addr):
    global blockSIZE, chanENABLE, blockCHANS, modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    vals=getBLOCK(addr)
    #print(vals)
    chanVALS=[None for z in range(16)]
    N=0
    for i in range(16):
        if (chanENABLE[addr][i]==True):
            N=N+1
    if (N>0):
        k=blockSIZE[addr]
        for i in range(k):
            chanVALS[blockCHANS[i]]=vals[i]
    return chanVALS

def startBLOCK(addr, num):
#def startBLOCK(addr, num, trig=None):
    global blockSIZE, blockMODE, ADCbusy, modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    ADCbusy[addr]=True
    assert ((num<=8192) and (num>0)), 'Block size has to be between 1 and 8,192 samples'
    blockSIZE[addr]=num
    #blockMODE[addr]=True
    num=num-1
    param1=num>>8
    param2=num&0xFF
    #param1=param1 + trigFlag
    ppCMD(addr,0x35,param1,param2,0)


def startSTREAM(addr, num):
    global blockSIZE, streamMODE, ADCbusy, modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    assert (streamMODE[addr]==False), "This ADCplate is already streaming data."
    assert ((num<=4096) and (num>0)), 'Block size for streaming has to be between 1 and 4,096 samples'
    blockSIZE[addr]=num
    streamMODE[addr]=True
    ADCbusy[addr]=True
    num=num-1
    param1=num>>8
    param2=num&0xFF
    ppCMD(addr,0x36,param1,param2,0)

def stopSTREAM(addr):
    global modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    assert (streamMODE[addr]==True), 'This ADCplate is not currently in stream mode.'
    streamMODE[addr]=False
    ADCbusy[addr]=False
    ppCMD(addr,0x37,0,0,0)

def getSTREAM(addr):
    global blockSIZE
    global blockVALS
    global blockBYTES
    global blockMODE
    global modeLIST
    global blockCHANS
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    if (streamMODE[addr]==False):
        print ('This ADCplate is not currently in stream mode.')
        return
    num=blockSIZE[addr]
    num=num-1
    param1=num>>8
    param2=num&0xFF
    fetchBLOCK(addr,0x38,param1,param2,blockSIZE[addr]<<2)
    #SF.binWRITE(addr,bytes(blockBYTES[0:(blockSIZE[addr]<<2)]))
    for i in range(blockSIZE[addr]):
        k=i<<2
        val=blockBYTES[k]<<16
        val += blockBYTES[k+1]<<8
        val += blockBYTES[k+2]
        lblockCHAN=blockBYTES[k+3] & 0x0F
        #type=chanTYPE[addr][blockCHANS[i]]
        #type=type[0].lower()
        #lastTYPE[addr]=type
        if (lblockCHAN<12):
            Vin=10.0*Vref*((val/8388608.0)-1)*FSR/25
        else:
            Vin=1000/50*Vref*val/16777216.0 #return current in mA
        blockVALS[i] = round(Vin,6)
        blockCHANS[i]=lblockCHAN
    return blockVALS[0:blockSIZE[addr]]
    
def getBLOCK(addr):
    global blockSIZE
    global blockVALS
    global blockBYTES
    global blockMODE
    global blockCHANS
    global ADCbusy, Vref, modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    if ((streamMODE[addr]==False) and (trigENABLED[addr]==False)):
        ADCbusy[addr]=False
    num=blockSIZE[addr]
    num=num-1
    param1=num>>8
    param2=num&0xFF
    #print (param1, param2, blockSIZE[addr])
    fetchBLOCK(addr,0x38,param1,param2,blockSIZE[addr]<<2)
    #blockMODE[addr]=False
    #Vref=2.5
    for i in range(blockSIZE[addr]):
        k=i<<2
        val=blockBYTES[k]<<16
        val += blockBYTES[k+1]<<8
        val += blockBYTES[k+2]
        lblockCHAN=blockBYTES[k+3] & 0x0F
        #type=chanTYPE[addr][blockCHANS[i]]
        #type=type[0].lower()
        #lastTYPE[addr]=type
        if (lblockCHAN<12):
            Vin=10.0*Vref*((val/8388608.0)-1)*FSR/25
        else:
            Vin=1000/50*Vref*val/16777216.0 #return current in mA
        blockVALS[i] = round(Vin,6)
        blockCHANS[i]=lblockCHAN
    return blockVALS[0:blockSIZE[addr]]

#==============================================================================#
# AD4112 Diagnostic Functions                                                  #
#==============================================================================#
def dumpCHANregs(addr,chan):
    global ADCbusy
    if (ADCbusy[addr]==True):
        print("Command ignored - this ADCplate is busy with a measurement.")
        return
    assert(chan >= 0 and chan <= 0xF), "invalid chan value"
    Channel=getADCreg(addr,0x10+chan)
    print ('CH'+str(chan)+':',hex(Channel))

    chan=(Channel>>12)&0x07

    # if (chan<8):
        # chan=chan>>1
    # else:
        # chan=chan-8
    Setup=getADCreg(addr,0x20+chan)
    print ('SETUPCON'+str(chan)+':',hex(Setup))
    Filter=getADCreg(addr,0x28+chan)
    print ('FILTCON'+str(chan)+':',hex(Filter))
    Offset=getADCreg(addr,0x30+chan)
    print ('OFFSET'+str(chan)+':',hex(Offset))
    Gain=getADCreg(addr,0x38+chan)
    print ('GAIN'+str(chan)+':',hex(Gain))
    Status=getADCreg(addr,0)
    print ('Status:',hex(Status))
    print ('-----------------')


#returns a single ADC register
def getADCreg(addr,reg):
    if (VerifyADDR(addr) == 0):
        return
    #assert(reg >= 0x10 and reg <= 0x3F), "invalid register address"
    if (reg==0):
        resp=ppCMD(addr,0x3E,reg,0,1)
        val=resp[0]
    if (reg>=0x10 and reg<0x30):
        resp=ppCMD(addr,0x3E,reg,0,2)
        val=(resp[0]<<8)+resp[1]
    if (reg>=0x30):
        resp=ppCMD(addr,0x3E,reg,0,4)
        val=(resp[1]<<16)+(resp[2]<<8)+resp[3]
    return val

#==============================================================================#
# Digital Input Functions                                                      #
#==============================================================================#
def getDINbit(addr,bit):
    if (VerifyADDR(addr) == 0):
        return
    assert ((bit>=0) and (bit<=4)),"DIN bit must be between 0 and 4"
    resp=ppCMD(addr,0x20,bit,0,1)
    if resp[0] > 0:
        return 1
    else:
        return 0
    pass

def getDINall(addr):
    if (VerifyADDR(addr) == 0):
        return
    resp=ppCMD(addr,0x25,0,0,1)
    return resp[0]

def enableDINevent(addr, bit):    # enable DIN interrupt
    if (VerifyADDR(addr) == 0):
        return
    assert ((bit>=0) and (bit<=3)),"DIN bit must be between 0 and 3"
    ppCMD(addr,0x21,bit,0,0)

def disableDINevent(addr,bit):    # disable DIN interrupt
    if (VerifyADDR(addr) == 0):
        return
    assert ((bit>=0) and (bit<=4)),"DIN bit must be between 0 and 4"
    ppCMD(addr,0x22,bit,0,0)

#===============================================================================#
# Event Functions                                                               #
#===============================================================================#
def enableEVENTS(addr,signal=None): #ADC will pull down on selected SRQ pin if an enabled event occurs
    global srqSOURCE
    if (VerifyADDR(addr) == 0):
        return
    if (signal==None):
        signal=srqSOURCE[addr]
    else:
        if ((signal!=SHARED) and (signal!=DEDICATED)):
            print("Command ignore - signal value must be SHARED or DEDICATED")
            return
        if (signal==DEDICATED):
            GPIO.setup(drMAP[addr], GPIO.IN, pull_up_down=GPIO.PUD_UP)            
        srqSOURCE[addr]=signal
    ppCMD(addr,0x04,signal,0,0)

def disableEVENTS(addr):   #ADC will not assert interrupts on INT pin (GPIO22) if an enabled event occurs
    if (VerifyADDR(addr) == 0):
        return
    ppCMD(addr,0x05,0,0,0)

def getEVENTS(addr):   #read INT flag register - this resets interrupt line and clears the register
    if (VerifyADDR(addr) == 0):
        return
    resp=ppCMD(addr,0x06,0,0,1)
    value=(resp[0])
    return value

# The ADCplate does not support the masking of interrupts - this must be done in SW.
# def configEVENTS(addr,pattern):    #set the pattern for the interrupt mask
    # if (VerifyADDR(addr) == 0):
        # return
    # if ((pattern<0) or (pattern>31)):
        # print("Command ignored - pattern value must be between 0 and 31")
        # return
    # ppCMD(addr,0x07,pattern,0,1)

def check4EVENTS(addr):
    global drMAP
    if (VerifyADDR(addr) == 0):
        return
    stat=False
    if (srqSOURCE[addr]==DEDICATED):
        if (GPIO.input(drMAP[addr])==0):
            stat=True
    else:
        if (GPIO.input(ppINT)==0):
            stat=True
    return stat

#==============================================================================#
# Trigger Functions - these functions work w/ Block and List ADC readings only #
#==============================================================================#
def configTRIG(addr,mode,primary=None):
    global modeLIST, trigENABLED
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    mode=mode.upper()
    if ((mode!='OFF') and (mode!='EXT') and  (mode!='SW') and (mode!='CLOCKED')):
        print('Command ignored -',mode,' is not a valid trigger mode.')
        return
    if (mode=='OFF'):
        disableTRIG(addr)
        ppCMD(addr,0x42,0,0,0)
        trigENABLED[addr]=False
    elif (mode=='EXT'):
        nmode=1
        ppCMD(addr,0x42,nmode,0,0)
        trigENABLED[addr]=True
    elif (mode=='SW'):
        trigENABLED[addr]=True
        nmode = 2
        if primary==True:
            nmode = nmode + 0x80
        ppCMD(addr,0x42,nmode,0,0)
    else:   #defaults to clocked mode
        trigENABLED[addr]=True
        nmode=3
        if primary==True:
            nmode = nmode + 0x80
        ppCMD(addr,0x42,nmode,0,0) #set for clocked internal trigger mode
                                   #default trigger frequency is 100Hz.

def startTRIG(addr):   #start trigger mode - must be used after trig configuration is set
    global modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    if (trigENABLED[addr] != True):
        print ('The trigger settings have to be set before executing this function. See configTRIG.')
        return
    ppCMD(addr,0x41,0,0,0)
    startSCAN(addr,True)
    trigENABLED[addr]=True

def stopTRIG(addr):  #disable trigger mode
    global modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    ppCMD(addr,0x40,0,0,0)
    trigENABLED[addr]=False

def triggerFREQ(addr,freq): #configure internal trigger clock and output a pulse if primary == True
    global modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    assert (freq<=1000),'The internal frequency should not exceed 1000Hz'
    val=int(64000/freq)-1
    actualFreq=64000/(val+1)
    ppCMD(addr,0x44,val>>8,val&0xFF,0)
    return actualFreq

def swTRIGGER(addr):    #trigger ADC measurement and output a pulse if primary == True
    global modeLIST
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    ppCMD(addr,0x43,0,0,0)


def maxTRIGfreq(addr):
    global modeLIST
    global chanTYPE
    global chanENABLE
    singleCHANrates=[1.25,2.5,5,10,16.667,20,25,50,59.98,100.2,200.3,381,504,1007,2597,5208,10417,15625,31250]
    multiCHANrates=[1.25,2.5,5,10,16.667,20,25,49.68,59.52,100.2,200.3,380.95,503.8,1007,2597,3115,4444,5181,6211]
    if (VerifyADDR(addr) == 0):
        return
    if (modeLIST[addr] != 3):
        print ('This command is only valid for ADVanced mode.')
        return
    chanCount=0
    for i in range(16):
        if (chanENABLE[addr][i]==True):
            chanCount = chanCount+1
    if (chanCount==0):
        print ('No channels enabled yet.')
        return
    sTime=0
    for i in range(16):
        if (chanENABLE[addr][i]==True):
            if (chanCount<2):
                sTime=sTime+1/singleCHANrates[chanTYPE[addr][i]]
            else:
                sTime=sTime+1/multiCHANrates[chanTYPE[addr][i]]
    return 1.0/sTime
                
        
    

#==============================================================================#
# LED Functions                                                                #
#==============================================================================#
def setLED(addr):
    if (VerifyADDR(addr) == 0):
        return
    ppCMD(addr,0x60,0,0,0)

def clrLED(addr):
    if (VerifyADDR(addr) == 0):
        return
    ppCMD(addr,0x61,0,0,0)

def toggleLED(addr):
    if (VerifyADDR(addr) == 0):
        return
    ppCMD(addr,0x62,0,0,0)

def getLED(addr):
    if (VerifyADDR(addr) == 0):
        return
    resp=ppCMD(addr,0x63,0,0,1)
    return resp[0]

#==============================================================================#
# System Functions                                                             #
#==============================================================================#
def getFWrev(addr):
    if (VerifyADDR(addr) == 0):
        return
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getHWrev(addr):
    if (VerifyADDR(addr) == 0):
        return
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return ADCversion

def setINT(addr):
    if (VerifyADDR(addr) == 0):
        return
    ppCMD(addr,0xF4,0,0,0)

def clrINT(addr):
    if (VerifyADDR(addr) == 0):
        return
    ppCMD(addr,0xF5,0,0,0)


#==============================================================================#
# Utility Functions                                                            #
#==============================================================================#
def getID(addr):
    global ADCbaseADDR
    global ppFRAME
    global DataGood
    if (VerifyADDR(addr) == 0):
        return ''
    addr=addr+ADCbaseADDR
    id=""
    arg = list(range(4))
    #resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)==1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True):
        GPIO.output(ppFRAME,True)
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        spi.xfer(arg,500000,5)
        #DataGood=True
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            time.sleep(0.000001)     #allow the uP some time to initialize the SPI
            count=0
            csum=0
            go=True
            while (go):
                dummy=spi.xfer([00],500000,5)
                if (dummy[0] != 0):
                    num = dummy[0]
                    csum += num
                    id = id + chr(num)
                    count += 1
                else:
                    dummy=spi.xfer([00],500000,1)
                    checkSum=dummy[0]
                    go=False
                if (count>25):
                    go=False
                    DataGood=False
            if ((~checkSum & 0xFF) != (csum & 0xFF)):
                DataGood=False
        GPIO.output(ppFRAME,False)
        time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return id

def VerifyADDR(addr):
    global MAXADDR, adcsPresent
    if((addr<0) or (addr>=MAXADDR)):
        print("ADCplate address out of range - must be between 0 and 7")
        return 0
    addr_str=str(addr)
    if (adcsPresent[addr]!=1):
        print("Command ignored - no ADCplate found at address "+addr_str)
        return 0
    return 1

def ppCMD(addr,cmd,param1,param2,bytes2return,slow=None):
    global ADCbaseADDR
    global DataGood
    if (slow==None):
        tOut=0.05
    else:
        tOut=3.0
    arg = list(range(4))
    resp = [0]*(bytes2return+1)
    arg[0]=addr+ADCbaseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)!=0):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    GPIO.output(ppFRAME,True)
    time.sleep(0.000001)     #allow the uP some time to initialize the SPI
    spi.xfer(arg,400000,0)
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):
            wait=False
        if ((time.time()-t0)>tOut):   #timeout
            wait=False
            DataGood=False
    if (bytes2return>0) and DataGood:
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):
                wait=False
            if ((time.time()-t0)>0.1):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            resp=spi.xfer([0]*(bytes2return+1),4000000,0)   #don't exceed 4Mhz
            csum=0;
            for i in range(0,bytes2return+1):
                csum+=resp[i]
            if ((csum & 0xFF) != 0xFF):
                DataGood=False
    GPIO.output(ppFRAME,False)
    time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return resp

def fetchBLOCK(addr,cmd,param1,param2,bytes2return):
    global ADCbaseADDR
    global DataGood
    global blockBYTES
    arg = list(range(4))
    arg[0]=addr+ADCbaseADDR
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)!=0):  #Ensure that ACK is high before proceeding
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    GPIO.output(ppFRAME,True)       #Set the FRAME signal high
    time.sleep(0.000001)            #allow the uP some time to initialize the SPI
    spi.xfer(arg,500000,0)     #Send the fetch command
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)==0):  #wait until the command is received and ACKnowledged
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if DataGood:
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        blockBYTES=spi.xfer3([0x55]*bytes2return,8000000,0)
    GPIO.output(ppFRAME,False)

def getADDR(i):
    global ADCbaseADDR, DataGood
    resp=ppCMD(i,0x00,0,0,1)
    if (DataGood):
        return resp[0]-ADCbaseADDR
    else:
        return 8

def quietPoll():
    global adcsPresent, drMAP
    ppFoundCount=0
    for i in range (0,8):
        adcsPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):
            adcsPresent[i]=1
            ppFoundCount += 1
            initADC(i)
            #disableEVENTS(i)

def RESET(addr):
    VerifyADDR(addr)
    ppCMD(addr,0x0F,0,0,0)
    time.sleep(1)
    quietPoll()

quietPoll()


import spidev
import time
import string
import site
import sys
from gpiozero import CPUTemperature
import RPi.GPIO as GPIO
GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("This library only works for Python3")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
POWERbaseADDR=0
ppFRAME = 25
ppINT = 22
ppACK = 23
ppSW = 24
GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppSW, GPIO.IN, pull_up_down=GPIO.PUD_UP)
spi = spidev.SpiDev()
spi.open(0,1)	
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/POWER24help.txt'
#helpPath='POWER24help.txt'       #for development only
POWER24version=1.0
DataGood=False
cpu = CPUTemperature()

GRN=0
RED=1

RMAX = 2000
MAXADDR=8

powerPresent = 0
powerType=0

def CLOSE():
	spi.close()
	GPIO.cleanup()

def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")
        
#==============================================================================#	
# A2D Functions	     	                                              		   #
#==============================================================================#	
def getVin(addr=None):
    resp=ppCMD(0,0x30,0,0,2)
    value=(256*resp[0]+resp[1])
    value=round((value*2.4*2.5/4095),3)
    return value   

def getIin(addr=None):
    if(powerType==24):
        print ("This function is not supported by the POWERplate24")
        return
    else:
        resp=ppCMD(0,0x30,1,0,2)
        value=(256*resp[0]+resp[1])
        value=round((value*2.4*2.0/4095),3)
        return value     

def getHVin(addr=None):
    if(powerType==24):
        resp=ppCMD(0,0x30,1,0,2)
        value=(256*resp[0]+resp[1])
        value=round((value*2.4*12.4573/4095),3)
    else:
        print ("This function is only supported by the POWERplate24")
        value=0        
    return value   

def getCPUtemp(addr=None):
    return cpu.temperature

def getADC(addr,channel):
    resp=ppCMD(0,0x30,channel,0,2)
    value=(256*resp[0]+resp[1])
    return value

#==============================================================================#	
# RTC and Schedule Functions                                                   #
#==============================================================================#
def setRTC(addr,zone):
    zone=zone.lower()
    assert ((zone=='l') or (zone=='g')),"Invalid zone - must be 'l' for local or 'g' for Greenwich"
    if (zone=='l'):
        lTime=time.localtime()
    else:
        lTime=time.gmtime()
    resp=ppCMD(0,0xD1,lTime.tm_hour,0,0)
    resp=ppCMD(0,0xD0,lTime.tm_min,lTime.tm_sec,0)
    
def setWAKE(addr,hr,min,sec):
    resp=ppCMD(0,0xD3,hr,0,0)
    resp=ppCMD(0,0xD2,min,sec,0)
    
def enableWAKE(addr=None):
    resp=ppCMD(0,0xD5,1,0,0)

def disableWAKE(addr=None):
    resp=ppCMD(0,0xD5,0,0,0)

def getWAKESOURCE(addr=None):
    resp=ppCMD(0,0x57,0,0,1)
    return resp[0]
 
#===============================================================================#	
# LED Functions	                                                   		   		#
#===============================================================================#			
def setLED(addr,led):
    led=led.upper()
    assert (led=='RED' or led=='GREEN' or led=='GRN' or led=='YELLOW' or led=='YEL' or led=='OFF'),"Invalid LED argument"
    resp=ppCMD(0,0x61,0,0,0) 
    resp=ppCMD(0,0x61,1,0,0)
    if (led=='RED' or led=='RD'):
        resp=ppCMD(0,0x60,1,0,0)
    if (led=='GREEN' or led=='GRN'):
        resp=ppCMD(0,0x60,0,0,0)
    if (led=='YELLOW' or led=='YEL'):
        resp=ppCMD(0,0x60,0,0,0) 
        resp=ppCMD(0,0x60,1,0,0)        

# ledMODE command
# mode 0: auto: always off
# mode 1: auto: blink
# mode 2: auto: always on (default)
# mode 3: manual
def ledMODE(addr,mode):
    assert (mode>=0 and mode<=3),"Invalid mode value. Must be 0, 1, 2, or 3"
    resp=ppCMD(0,0x6F,mode,0,0)
  
def VerifyLED(led):
    assert (led>=0 and led<=1),"Invalid LED value. Must be 0 or 1"

#==============================================================================#	
# Switch and Fan Functions	                                                   #
#==============================================================================#   
def fanOFF(addr=None):
    resp=ppCMD(0,0xEE,0,0,0)
    
def fanON(addr=None):
    resp=ppCMD(0,0xEF,0,0,0)
    
def fanSTATE(addr=None):
    resp=ppCMD(0,0xED,0,0,1)
    return resp[0]

def getSWstate(addr=None):
    resp=ppCMD(0,0x50,0,0,1)
    return resp[0]    

def setSHUTDOWNdelay(addr,delay):
    assert (delay>=10 and delay<=240),"Invalid delay value. Must be between 10 and 240"
    resp=ppCMD(0,0x55,delay,0,0)

def enablePOWERSW(addr, bypass=None):
    if (getFWrev(addr)>=1.2):
        if (bypass==None):
            bypass=True
        else:
            assert(bypass==True or bypass==False), 'If used, bypass argument must be "True" or "False".'
    else:
        bypass=False
    if(bypass):
        bparg=1
    else:
        bparg=0   
    resp=ppCMD(0,0x53,bparg,0,0)

def disablePOWERSW(addr=None):
    resp=ppCMD(0,0x54,0,0,0)    

def powerOFF(addr=None):
    resp=ppCMD(0,0x56,0,0,0)
    
#==============================================================================#	
# POWER Status Functions - only for POWERplate24                               #
#==============================================================================# 
 
def statEnable(addr=None):	#POWERplate24 will pull down on STAT pin if an enabled event occurs
    if(powerType==24):
        resp=ppCMD(addr,0x04,0,0,0)
    else:
        print ("This function is only supported by the POWERplate24")       
	
def statDisable(addr=None):   #POWERplate24 will not assert STAT
    if(powerType==24):
        resp=ppCMD(addr,0x05,0,0,0)
    else:
        print ("This function is only supported by the POWERplate24") 
    
def getPOWchange(addr=None):	#read Power status change register in POWERplate24 - this clears stat change line and the register
    if(powerType==24):
        resp=ppCMD(addr,0x06,0,0,1)
        value=(resp[0])
    else:
        print ("This function is only supported by the POWERplate24")
        value=0
    return value 
    
def getPOWstatus(addr=None):	#read curent power status register in POWERplate24
    if(powerType==24):
        resp=ppCMD(addr,0x07,0,0,1)
        value=(resp[0])
    else:
        print ("This function is only supported by the POWERplate24")
        value=0
    return value 

def getSTATflag(addr=None):
    if(powerType==24):
        if (GPIO.input(ppINT)==1):
            value=False
        else:
            value=True
    else:
        print ("This function is only supported by the POWERplate24")
        value=0
    return value 
 
#==============================================================================#	
# LOW Level Functions	                                                       #
#==============================================================================#          
def VerifyAINchannel(ain):
    assert ((ain>=0) and (ain<=1)),"Analog input channel value out of range. Must be in the range of 0 to 1"    

def VerifyADDR(addr):
    assert (addr==0),"POWERplate address out of range"
    addr_str=str(addr)
    assert (powerPresent==1),"No POWERplate found"
    
def ppCMD(addr,cmd,param1,param2,bytes2return):
    global POWERbaseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = []
    arg[0]=0;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)==1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,1000000,1)
    #DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):              
                wait=False
            if ((time.time()-t0)>0.08):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            #time.sleep(.0001)
            for i in range(0,bytes2return+1):	
                dummy=spi.xfer([00],1000000,5)
                resp.append(dummy[0])
            csum=0;
            for i in range(0,bytes2return):
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    #time.sleep(.001)
    GPIO.output(ppFRAME,False)
    #time.sleep(.001)
    return resp
 
def getID(addr=None):
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=0;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)==1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (DataGood==True):        
        GPIO.output(ppFRAME,True)
        null=spi.xfer(arg,1000000,1)
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):              
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            count=0 
            csum=0
            go=True
            while (go): 
                dummy=spi.xfer([00],500000,40)
                if (dummy[0] != 0):
                    num = dummy[0]
                    csum += num
                    id = id + chr(num)
                    #print count, num
                    count += 1
                else:
                    dummy=spi.xfer([00],500000,40)  
                    checkSum=dummy[0]                
                    go=False 
                if (count>25):
                    go=False
                    DataGood=False
            #print checkSum, ~checkSum & 0xFF, csum & 0xFF
            if ((~checkSum & 0xFF) != (csum & 0xFF)):
                DataGood=False
        GPIO.output(ppFRAME,False)
    return id

def getFWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0
	
def getHWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return POWER24version
    
def readFLASH(addr,flashadddr):
    global POWERbaseADDR
    p1=flashadddr>>8
    p2=flashadddr&0xFF
    resp=ppCMD(0,0xFE,p1,p2,1)
    return resp[0]


def getADDR():
    global POWERbaseADDR
    resp=ppCMD(0,0x00,0,0,1)
    #print resp, DataGood;
    if (DataGood):
        return resp[0]        
    else:
        return 8
    
def quietPoll():   
    global powerPresent, powerType
    rtn = getADDR()
    if (rtn==0):           
        powerPresent=1
        id=getID()
        if (id.find('24')>0):
            powerType=24      
        setRTC(0,'l')       

def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x0F,0,0,0)    
    time.sleep(1)
    quietPoll()

quietPoll()          


import spidev
import time
import string
import site
import sys
from numbers import Number
import RPi.GPIO as GPIO
from six.moves import input as raw_input
GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (2,7,0)):
    sys.stderr.write("You need at least python 2.7.0 to use this library")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
MOTORbaseADDR=16
ppFRAME = 25
ppINT = 22
GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)
spi = spidev.SpiDev()
spi.open(0,1)	
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/MOTORhelp.txt'
#helpPath='MOTORhelp.txt'
MPversion=1.4
# Version 1.4 - made help() function compatible with Python 2 and 3. Fixed getID function
# Version 1.3 - adjusted timing on command functions to compensate for RPi SPI changes
RMAX = 2000
MAXADDR=8
motorsPresent = list(range(8))

def CLOSE():
	spi.close()
	GPIO.cleanup()

def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print(s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't open help file.")

def Version():
    print('MOTORplate Python Module Version '+str(MPversion))

    
#==============================================================================#	
# Stepper Motor Functions	                                                   #
#==============================================================================#    
def stepperCONFIG(addr,motor,dir,resolution,rate,acceleration):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect stepper motor selection"   
    dir=dir.lower()
    if ((dir!='ccw') and (dir!='cw')):
        return "ERROR: incorrect direction parameter" 
    resolution=parseRES(resolution)
    if ((resolution>3) or (resolution<0)):
        return "ERROR: incorrect resolution value"
    if ((rate>RMAX) or (rate<1)):
        return "ERROR: incorrect rate value" 
    if ((acceleration>10) or (acceleration<0)):
        return "ERROR: incorrect acceleration time"         
    ## Param1:|0|DIR|RES1|RES2|NA|RATE10|RATE9|RATE8|        
    param1=0
    if dir=='cw':
        param1=0x40
    param1=param1+(resolution<<4)
    param1=param1+(rate>>8)
    ## Param2:|RATE7-RATE0|     
    param2=rate & 0x00FF
    cmd=0x10
    if (motor=='b'):
        cmd=0x11
    ppCMDm(addr,cmd,param1,param2,0)  
    time.sleep(.001)   ##Allow uP on board time to process 
    ## Send 2nd set of parameters but with same command numbers 
    ## to add acceleration increment. This is a 15 bit number with the upper
    ## 5 bits being the integer value and the lower 10 bits being the fractional
    ## part
    if (acceleration==0):
        increment=0
    else:
        increment=int(1024*rate/(acceleration*RMAX)+0.5)
    ## Param1:|1|ACC14|ACC13|ACC12|ACC11|ACC10|ACC9|ACC10| 
    param1=0x80+(increment>>8)
    param2=increment & 0x00FF
    cmd=0x10
    if (motor=='b'):
        cmd=0x11
    ppCMDm(addr,cmd,param1,param2,0) 
    time.sleep(.0001)   ##Allow uP on board time to process 
    
    
def stepperMOVE(addr,motor,steps):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1        
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection"  
    if (steps>65535):
        return "ERROR: step count greater than 65,535"
    cmd=0x12
    if (motor=='b'):
        cmd=0x13    
    param1 = steps>>8
    param2 = steps&0xFF
    ppCMDm(addr,cmd,param1,param2,0)
#    time.sleep(.001)   ##Allow uP on board time to process 
    
    
def stepperJOG(addr,motor):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection"   
    cmd=0x14
    if (motor=='b'):
        cmd=0x15  
    ppCMDm(addr,cmd,0,0,0)
#    time.sleep(.001)   ##Allow uP on board time to process 
    
 
def stepperSTOP(addr,motor):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection" 
    cmd=0x16
    if (motor=='b'):
        cmd=0x17  
    ppCMDm(addr,cmd,0,0,0)

def stepperRATE(addr,motor,rate):    ## function to change step rate of jogging or stopped stepper
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection" 
    if ((rate>RMAX) or (rate<1)):
        return "ERROR: incorrect rate value" 
    ## Param1:|0|0|0|0|0|RATE10|RATE9|RATE8|
    param1=(rate>>8)
    ## Param2:|RATE7-RATE0|     
    param2=rate & 0x00FF       
    cmd=0x18
    if (motor=='b'):
        cmd=0x19  
    ppCMDm(addr,cmd,param1,param2,0)
#    time.sleep(.001)   ##Allow uP on board time to process 
   
def stepperOFF(addr,motor):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection" 
    cmd=0x1E
    if (motor=='b'):
        cmd=0x1F  
    ppCMDm(addr,cmd,0,0,0)
   
def parseRES(res):
    if (isinstance(res,Number)):
        return res
    res=res.lower()
    if (res=='f'):
        return 0
    if (res=='h'):
        return 1    
    if (res=='m4'):
        return 2
    if (res=='m8'):
        return 3   
    return -1    
#==============================================================================#	
# DC Motor Functions	                                                       #
#==============================================================================#     
def dcCONFIG(addr,motor,dir,speed,acceleration):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection. Must be between 1 and 4."   
    dir=dir.lower()
    if ((dir!='ccw') and (dir!='cw')):
        return "ERROR: incorrect direction parameter. Must be ''cw'' or ''ccw''." 
    if ((speed>100)or(speed<0)):
        return "ERROR: speed out of range. Must be between 0 and 100"    
    if ((acceleration>10) or (acceleration<0)):
        return "ERROR: incorrect acceleration time"   
       
    speed = int((speed*1023/100)+0.5)  
    if ((motor==1)or(motor==2)):
        speed=(speed*5)>>3
    ## Param1:|motor num 1|motor num 0|NA|direction|NA|NA|dc9|dc8|     
    param1=(motor-1)<<6
    if dir=='cw':
        param1=param1+0x10
    param1=param1+(speed>>8)
    param2= speed & 0x00FF
    ppCMDm(addr,0x30,param1,param2,0)  
    time.sleep(.001)   ##Allow uP on board time to process 
    ## Send 2nd set of parameters but with same command numbers 
    ## to add acceleration increment. This is a 15 bit number with the upper
    ## 5 bits being the integer value and the lower 10 bits being the fractional
    ## part
    if (acceleration==0):
        increment=0
    else:
        increment=int(1024*speed/(acceleration*RMAX)+0.5)
    ## Param1:|0|ACC14|ACC13|ACC12|ACC11|ACC10|ACC9|ACC10| 
    param1=(increment>>8)
    param2=increment & 0x00FF
    ppCMDm(addr,0x3A+(motor-1),param1,param2,0)
    time.sleep(.001)   ##Allow uP on board time to process
    
def dcSPEED(addr,motor,speed):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection. Must be between 1 and 4."   
    if ((speed>100)or(speed<0)):    
        return "ERROR: speed out of range. Must be between 0 and 100" 
    speed = int((speed*1023/100)+0.5)  
    if ((motor==1) or (motor==2)):
        speed=(speed*5)>>3
    ## Param1:|motor num 1|motor num 0|NA|direction|NA|NA|dc9|dc8|     
    param1=(motor-1)<<6
    param1=param1+(speed>>8)
    param2= speed & 0x00FF       
    ppCMDm(addr,0x33,param1,param2,0)    
        
def dcSTART(addr,motor):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection" 
    ppCMDm(addr,0x31,motor-1,0,0)
        
def dcSTOP(addr,motor):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection" 
    ppCMDm(addr,0x32,motor-1,0,0)
    
#==============================================================================#	
# Sensor Functions	                                                           #
#==============================================================================#    
def getSENSORS(addr):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    resp=ppCMDm(addr,0x20,0,0,1)
    return resp[0]        
        
def getTACHcoarse(addr,tachnum):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((tachnum>4) or (tachnum<1)):
        return "ERROR: tachometer number must be between 1 and 4."
    resp=ppCMDm(addr,0x22,tachnum,0,2)
    return resp[0]*256+resp[1]

def getTACHfine(addr,tachnum):
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((tachnum>4) or (tachnum<1)):
        return "ERROR: tachometer number must be between 1 and 4."
    resp=ppCMDm(addr,0x23,tachnum,0,2)
    return resp[0]*256+resp[1]

#==============================================================================#	
# LED Functions	                                                               #
#==============================================================================#   
def setLED(addr):
	if (addr>MAXADDR):
		return "ERROR: address out of range - must be less than", MAXADDR-1
	resp=ppCMDm(addr,0x60,0,0,0)

def clrLED(addr):
	if (addr>MAXADDR):
		return "ERROR: address out of range - must be less than", MAXADDR-1	
	resp=ppCMDm(addr,0x61,0,0,0)

def toggleLED(addr):
	if (addr>MAXADDR):
		return "ERROR: address out of range - must be less than", MAXADDR-1	
	resp=ppCMDm(addr,0x62,0,0,0)

     
#==============================================================================#	
# Interrupt Control Functions	                                               #
#==============================================================================#	
def intEnable(addr):	#MOTORplate will pull down on INT pin if an enabled event occurs
	if (addr>MAXADDR):
		return "ERROR: address out of range - must be less than", MAXADDR-1
	resp=ppCMDm(addr,0x04,0,0,0)
	
def intDisable(addr):
	if (addr>MAXADDR):
		return "ERROR: address out of range - must be less than", MAXADDR-1
	resp=ppCMDm(addr,0x05,0,0,0)
	
def getINTflag0(addr):	            #read INT flag register0 in MOTORplate
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    resp=ppCMDm(addr,0x06,0,0,1)
    return resp[0]  
    
def getINTflag1(addr):	            #read INT flag register1 in MOTORplate
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    resp=ppCMDm(addr,0x07,0,0,1)
    return resp[0] 

def setSENSORint(addr,sensor):
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((sensor>4) or (sensor<1)):
        return "ERROR: sensor number must be between 1 and 4."
    ppCMDm(addr,0x24,sensor,0,0)        

def clrSENSORint(addr,sensor):
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((sensor>4) or (sensor<1)):
        return "ERROR: sensor number must be between 1 and 4."
    ppCMDm(addr,0x25,sensor,0,0)
    
def enablestepSTOPint(addr,motor):   #enable the interrupt at the end of a move
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection" 
    cmd=0x1A
    if (motor=='b'):
        cmd=0x1B 
    ppCMDm(addr,cmd,0,0,0)

def disablestepSTOPint(addr,motor):   #disable the interrupt at the end of a move
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection"   
    cmd=0x1C
    if (motor=='b'):
        cmd=0x1D 
    ppCMDm(addr,cmd,0,0,0)

def enablestepSTEADYint(addr,motor):   #enable interrupt when speed is stable
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection" 
    cmd=0x4A
    if (motor=='b'):
        cmd=0x4B 
    ppCMDm(addr,cmd,0,0,0)

def disablestepSTEADYint(addr,motor):   #disable the interrupt when speed is stable
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    motor=motor.lower()
    if ((motor!='a') and (motor!='b')):
        return "ERROR: incorrect motor selection"   
    cmd=0x4C
    if (motor=='b'):
        cmd=0x4D 
    ppCMDm(addr,cmd,0,0,0)

def enabledcSTOPint(addr,motor):   #enable the interrupt at the end of a move
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection" 
    ppCMDm(addr,0x34,motor-1,0,0)

def disabledcSTOPint(addr,motor):   #disable the interrupt at the end of a move
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection" 
    ppCMDm(addr,0x35,motor-1,0,0)

def enabledcSTEADYint(addr,motor):   #enable interrupt when speed is stable
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection" 
    ppCMDm(addr,0x36,motor-1,0,0)

def disabledcSTEADYint(addr,motor):   #disable the interrupt when speed is stable
    if (addr>=MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    if ((motor<1) or (motor>4)):
        return "ERROR: incorrect DC motor selection" 
    ppCMDm(addr,0x37,motor-1,0,0)

#==============================================================================#	
# System Functions	                                                           #
#==============================================================================#	    
def Poll():
    ppFoundCount=0
    for i in range (0,8):
        rtn = getADDR(i)
        if (rtn==i):
            print("MOTORplate found at address",rtn)
            ppFoundCount += 1
    if (ppFoundCount == 0):
        print("No MOTORplates found")    

def quietPoll():   
    global motorsPresent
    ppFoundCount=0
    for i in range (0,8):
        motorsPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):
            motorsPresent[i]=1
            ppFoundCount += 1
       
        
def getID(addr):
    global MOTORbaseADDR
    addr=addr+MOTORbaseADDR
    if (addr>255):
        return "ERROR: address out of range - must be less than 255"
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;

    ppFRAME = 25
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,300000,60)
    #null = spi.writebytes(arg)
    count=0
    time.sleep(.01)
    while (count<20): 
        dummy=spi.xfer([00],300000,20)
        dummy=spi.xfer([00],300000,20)
        if (dummy[0] != 0):
            num = dummy[0]
            id = id + chr(num)
            count = count + 1
        else:
            count=20
    GPIO.output(ppFRAME,False)
    return id

def getHWrev(addr):
    global MOTORbaseADDR
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    resp=ppCMDm(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0	 
    
def getFWrev(addr):
    global MOTORbaseADDR
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    resp=ppCMDm(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return MPversion    
    
def getADDR(addr):
    global MOTORbaseADDR
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    resp=ppCMDm(addr,0x00,0,0,1)
    return resp[0]-MOTORbaseADDR
    
def RESET(addr):
    global MOTORbaseADDR
    if (addr>MAXADDR):
        return "ERROR: address out of range - must be less than", MAXADDR-1
    resp=ppCMDm(addr,0x0F,0,0,0)    
    time.sleep(.10)
    
def ppCMDm(addr,cmd,param1,param2,bytes2return):
    global MOTORbaseADDR
    arg = list(range(4))
    resp = []
    arg[0]=addr+MOTORbaseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,300000,60)
    #null = spi.writebytes(arg)
    if bytes2return>0:
        time.sleep(.0001)
        for i in range(0,bytes2return):	
            dummy=spi.xfer([00],500000,20)
            resp.append(dummy[0])
    time.sleep(.001)
    GPIO.output(ppFRAME,False)
    time.sleep(.001)
    return resp	   

quietPoll()    


import spidev
import time
import site
import sys
import threading
import RPi.GPIO as GPIO
from six.moves import input as raw_input

GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("This library requires Python3")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
DIGIbaseADDR=0x58
ppFRAME = 25
ppACK = 23
ppSRQ=22

GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppSRQ, GPIO.IN, pull_up_down=GPIO.PUD_UP)

try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")    
    
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/DIGIhelp.txt'
#helpPath='DIGIhelp.txt'       #for development only
DIGIversion=1.0
# Version 1.0   -   initial release

RMAX = 2000
MAXADDR=8
digisPresent = list(range(8))
DataGood=False
lock = threading.Lock()
lock.acquire()

#==============================================================================#
# HELP Functions                                                               #
#==============================================================================#
def Help():
    help()

def HELP():
    help()  
    
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")

def getSWrev():
    return DIGIversion

#===============================================================================#   
# Digital Input Functions                                                       #
#===============================================================================#
def getDINbit(addr,bit):
    VerifyADDR(addr)
    VerifyDINchannel(bit-1)
    resp=ppCMD(addr,0x20,bit-1,0,1)
    if resp[0] > 0:
        return 1
    else:
        return 0
        
def getDINall(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x25,0,0,1)
    return resp[0]

#==============================================================================#
# Event Functions                                                              #
#==============================================================================#
def enableDINevent(addr, bit, edge):  # enable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit-1)
    bit = bit-1
    if ((edge=='f') or (edge=='F')):
        resp=ppCMD(addr,0x21,bit,0,0)
    if ((edge=='r') or (edge=='R')):
        resp=ppCMD(addr,0x22,bit,0,0)
    if ((edge=='b') or (edge=='B')):
        resp=ppCMD(addr,0x23,bit,0,0)

def disableDINevent(addr,bit):    # disable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit-1)
    resp=ppCMD(addr,0x24,bit-1,0,0)    

def eventEnable(addr):    #DIGIplate will pull down on INT pin if an enabled event occurs
    VerifyADDR(addr)
    resp=ppCMD(addr,0x04,0,0,0)

def eventDisable(addr):   #DIGIplate will not assert interrupts
    VerifyADDR(addr)
    resp=ppCMD(addr,0x05,0,0,0)
    
def getEVENTS(addr):  #read INT flag register in DIGIplate - this clears interrupt line and the register
    VerifyADDR(addr)
    resp=ppCMD(addr,0x06,0,0,2)
    value=((resp[0]<<8) + resp[1])
    return value

def check4EVENTS():
    stat=False
    if (GPIO.input(ppSRQ)==0):
        stat=True
    return stat
    
#==============================================================================#    
# FREQ Functions                                                               #
#==============================================================================#      
def getFREQ(addr,chan):
    global DataGood
    VerifyADDR(addr)
    assert ((chan>=1) and (chan<=6)),"Frequency input channel value out of range. Must be in the range of 1 to 6"
    freq=0
    resp=ppCMD(addr,0xC0,0,chan-1,2) #get the upper 16 bits
    #print (1, DataGood, (resp[0]<<24)+(resp[1]<<16), resp[2])
    if(DataGood):
        counts=(resp[0]<<24)+(resp[1]<<16)
        resp=ppCMD(addr,0xC0,1,chan-1,2) #get the lower 16 bits
        #print (2, DataGood, (resp[0]<<8)+resp[1], resp[2])
        if (DataGood):
            counts=counts+(resp[0]<<8)+resp[1]
            if (counts>0):
                freq=1000000.0/counts
            else:
                freq=0
    return round(freq,3)

def getFREQall(addr):
    global DataGood
    VerifyADDR(addr)
    freqList=6*[0]
    for i in range(6):
        chan=i+1
        freq=0
        resp=ppCMD(addr,0xC0,0,chan-1,2) #get the upper 16 bits
        if(DataGood):
            counts=(resp[0]<<24)+(resp[1]<<16)
            resp=ppCMD(addr,0xC0,1,chan-1,2) #get the lower 16 bits
            #print (2, DataGood, (resp[0]<<8)+resp[1], resp[2])
            if (DataGood):
                counts=counts+(resp[0]<<8)+resp[1]
                if (counts>0):
                    freq=1000000.0/counts
                else:
                    freq=0
                freqList[i]=round(freq,3)
    return freqList    
    
    
#==============================================================================#    
# LED Functions                                                                #
#==============================================================================#   
def setLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x60,0,0,0)

def clrLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x61,0,0,0)

def toggleLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x62,0,0,0)
  
#==============================================================================#    
# SYSTEM Functions                                                             #
#==============================================================================#     
def getID(addr):
    global DIGIbaseADDR
    global ppFRAME
    global DataGood
    VerifyADDR(addr)
    addr=addr+DIGIbaseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)==1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False     
    if (DataGood==True):
        GPIO.output(ppFRAME,True)
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        null=spi.xfer(arg,500000,5)
        #DataGood=True
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            time.sleep(0.000001)     #allow the uP some time to initialize the SPI
            count=0 
            csum=0
            go=True
            while (go): 
                dummy=spi.xfer([00],500000,5)
                if (dummy[0] != 0):
                    num = dummy[0]
                    csum += num
                    id = id + chr(num)
                    count += 1
                else:
                    dummy=spi.xfer([00],500000,1)
                    checkSum=dummy[0]
                    go=False 
                if (count>25):
                    go=False
                    DataGood=False
            if ((~checkSum & 0xFF) != (csum & 0xFF)):
                DataGood=False
        GPIO.output(ppFRAME,False)
        time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return id

def getHWrev(addr):
    global DIGIbaseADDR
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0  
    
def getFWrev(addr):
    global DIGIbaseADDR
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return DIGIversion
        
#==============================================================================#    
# LOW Level Functions                                                          #
#==============================================================================#          
def VerifyDINchannel(din):
    assert ((din>=0) and (din<=7)),"Digital input channel value out of range. Must be in the range of 1 to 8"
    
def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"DIGIplate address out of range"
    addr_str=str(addr)
    assert (digisPresent[addr]==1),"No DIGIplate found at address "+addr_str


def ppCMD(addr,cmd,param1,param2,bytes2return):
    global DIGIbaseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    #resp = []
    resp = [0]*(bytes2return+1)
    arg[0]=addr+DIGIbaseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    wait=True
    t0=time.time()
    while(wait):    #ensure that ACK is high before asserting FRAME
        if (GPIO.input(ppACK)!=0):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False     
    GPIO.output(ppFRAME,True)       #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)     #Send out 4 byte command - ignore what comes back 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):  #wait up to 50msec for the addressed plate to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05): #test for timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If plate is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data     
            resp=spi.xfer([0]*(bytes2return+1),500000,0)    
            csum=0;                             
            for i in range(0,bytes2return):     #calculate and verify checksum
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    GPIO.output(ppFRAME,False)
    return resp

def setINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF4,0,0,0)

def clrINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF5,0,0,0)
    
def getADDR(addr):
    global DIGIbaseADDR
    resp = []
    resp=ppCMD(addr,0x00,0,0,1)
    return resp[0]-DIGIbaseADDR

    
def quietPoll():   
    global digisPresent
    ppFoundCount=0
    for i in range (0,8):
        digisPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            digisPresent[i]=1
            ppFoundCount += 1
            #RESET(i)

def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x0F,0,0,0)    
    time.sleep(.10)

quietPoll()    



import os
import sys
import time
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
from BASE import *


t=time.time()
bTime = [[t for x in range(8)] for y in range(8)]
bLast = [[0 for x in range(8)] for y in range(8)]
BTHROTTLE=0.05

def getBUTTON(addr,chan):
    VerifyADDR(addr)
    VerifyCHAN(chan)
    chan = chan-1
    if ((time.time()-bTime[addr][chan]) >= BTHROTTLE):
        bTime[addr][chan]=time.time()
        res=ppCMD(addr,0x2A,chan,0,1)
        bLast[addr][chan]=res[0]
    return bLast[addr][chan]
    
        
    
